from datetime import datetime

from django import forms
from django.utils.translation import ugettext_lazy as _
from django.utils.safestring import mark_safe
from django.core.urlresolvers import reverse
from django.template.defaultfilters import slugify

from captcha.fields import CaptchaField
from tendenci.core.categories.models import Category
from tendenci.addons.jobs.models import Job
from tendenci.core.perms.forms import TendenciBaseForm
from tinymce.widgets import TinyMCE
from tendenci.core.base.fields import SplitDateTimeField, EmailVerificationField
from tendenci.addons.jobs.models import JobPricing
from tendenci.addons.jobs.utils import get_payment_method_choices, pricing_choices
from tendenci.apps.user_groups.models import Group


request_duration_defaults = {
    'help_text': mark_safe('<a href="%s">Add pricing options</a>' % '/jobs/pricing/add/')
}

DURATION_CHOICES = (
    (14, '14 Days from Activation date'),
    (30, '30 Days from Activation date'),
    (60, '60 Days from Activation date'),
    (90, '90 Days from Activation date'),
    (120, '120 Days from Activation date'),
    (180, '180 Days from Activation date'),
    (365, '365 Days from Activation date'),
)


STATUS_DETAIL_CHOICES = (
    ('active', 'Active'),
    ('inactive', 'Inactive'),
    ('pending', 'Pending'),
    ('paid - pending approval', 'Paid - Pending Approval'),
)

STATUS_CHOICES = (
    (1, 'Active'),
    (0, 'Inactive'),
)


class JobForm(TendenciBaseForm):

    description = forms.CharField(required=False,
        widget=TinyMCE(attrs={'style': 'width:100%'},
        mce_attrs={'storme_app_label': Job._meta.app_label,
        'storme_model': Job._meta.module_name.lower()}))

    captcha = CaptchaField()
    
    start_dt = SplitDateTimeField(required=False, label=_('Position starts on:'),
        initial=datetime.now())

    activation_dt = SplitDateTimeField(label=_('Activation Date/Time'),
        initial=datetime.now())

    post_dt = SplitDateTimeField(label=_('Post Date/Time'),
        initial=datetime.now())

    expiration_dt = SplitDateTimeField(label=_('Expiration Date/Time'),
        initial=datetime.now())

    status_detail = forms.ChoiceField(
        choices=(('active', 'Active'), ('inactive', 'Inactive'), ('pending', 'Pending'),))

    list_type = forms.ChoiceField(initial='regular', choices=(('regular', 'Regular'),
                                                              ('premium', 'Premium'),))
    payment_method = forms.CharField(error_messages={'required': 'Please select a payment method.'})
    
    contact_email = EmailVerificationField(label=_("Contact email"), required=False)

    group = forms.ModelChoiceField(queryset=Group.objects.filter(status=True, status_detail="active"), required=True, empty_label=None)

    pricing = forms.ModelChoiceField(label=_('Requested Duration'), 
                    queryset=JobPricing.objects.filter(status=True).order_by('duration'))

    class Meta:
        model = Job
        fields = (
        'title',
        'slug',
        'description',
        'group',
        'code',
        'location',
        'skills',
        'experience',
        'education',
        'level',
        'period',
        'is_agency',
        'contact_method',
        'position_reports_to',
        'salary_from',
        'salary_to',
        'computer_skills',
        'tags',
        'pricing',
        'list_type',
        'start_dt',
        'activation_dt',
        'post_dt',
        'expiration_dt',
        'job_url',
        'entity',
        'contact_company',
        'contact_name',
        'contact_address',
        'contact_address2',
        'contact_city',
        'contact_state',
        'contact_zip_code',
        'contact_country',
        'contact_phone',
        'contact_fax',
        'contact_email',
        'contact_website',
        'tags',
        'allow_anonymous_view',
        'syndicate',
        'status',
        'status_detail',
        'payment_method',
       )

        fieldsets = [('Job Information', {
                      'fields': ['title',
                                'slug',
                                'description',
                                'group',
                                'job_url',
                                'start_dt',
                                'code',
                                'location',
                                'skills',
                                'computer_skills',
                                'experience',
                                'education',
                                'level',
                                'period',
                                'contact_method',
                                'position_reports_to',
                                'salary_from',
                                'salary_to',
                                'is_agency',
                                'tags',
                                'pricing',
                                'activation_dt',
                                'expiration_dt',
                                'post_dt',
                                'entity'
                                 ],
                      'legend': ''
                      }),
                      ('Payment', {
                      'fields': ['list_type',
                                 'payment_method'
                                 ],
                        'classes': ['payment_method'],
                      }),
                      ('Contact', {
                      'fields': ['contact_company',
                                'contact_name',
                                'contact_address',
                                'contact_address2',
                                'contact_city',
                                'contact_state',
                                'contact_zip_code',
                                'contact_country',
                                'contact_phone',
                                'contact_fax',
                                'contact_email',
                                'contact_website'
                                 ],
                        'classes': ['contact'],
                      }),
                     ('Security Code', {
                      'fields': ['captcha',
                                 ],
                        'classes': ['captcha'],
                      }),
                      ('Permissions', {
                      'fields': ['allow_anonymous_view',
                                 'user_perms',
                                 'member_perms',
                                 'group_perms',
                                 ],
                      'classes': ['permissions'],
                      }),
                     ('Administrator Only', {
                      'fields': ['syndicate',
                                 'status',
                                 'status_detail'],
                      'classes': ['admin-only'],
                    })]

    def __init__(self, *args, **kwargs):
        super(JobForm, self).__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields['description'].widget.mce_attrs['app_instance_id'] = self.instance.pk
            #self.fields['pricing'].initial = JobPricing.objects.filter(duration=self.instance.requested_duration)[0]
            if self.user.profile.is_superuser:
                self.fields['status_detail'].choices = STATUS_DETAIL_CHOICES
        else:
            self.fields['description'].widget.mce_attrs['app_instance_id'] = 0
            
        self.fields['pricing'].choices = pricing_choices(self.user)

        if 'payment_method' in self.fields:
            self.fields['payment_method'].widget = forms.RadioSelect(choices=get_payment_method_choices(self.user))
        
        # adjust fields depending on user status
        fields_to_pop = []
        if not self.user.is_authenticated():
            fields_to_pop += [
                'entity',
                'allow_anonymous_view',
                'user_perms',
                'group_perms',
                'member_perms',
                'post_dt',
                'activation_dt',
                'expiration_dt',
                'syndicate',
                'status',
                'status_detail'
            ]
        else:
            fields_to_pop += [
               'captcha'
            ]

        if not self.user.profile.is_superuser:
            fields_to_pop += [
                'slug',
                'entity',
                'group',
                'allow_anonymous_view',
                'user_perms',
                'member_perms',
                'group_perms',
                'post_dt',
                'activation_dt',
                'expiration_dt',
                'syndicate',
                'status',
                'status_detail'
            ]

        for f in list(set(fields_to_pop)):
            if f in self.fields:
                self.fields.pop(f)
                
    def save(self, *args, **kwargs):
        """
        Assigns the requested_duration of a job based on the
        chosen pricing.
        """
        job = super(JobForm, self).save(commit=False)
        if self.cleaned_data.has_key('pricing'):
            job.requested_duration = self.cleaned_data['pricing'].duration
        if kwargs['commit']:
            job.save()
        return job


class JobPricingForm(forms.ModelForm):
    duration = forms.ChoiceField(initial=14, choices=DURATION_CHOICES)
    status = forms.ChoiceField(initial=1, choices=STATUS_CHOICES, required=False)

    class Meta:
        model = JobPricing
        fields = (
            'title',
            'duration',
            'regular_price',
            'premium_price',
            'regular_price_member',
            'premium_price_member',
            'show_member_pricing',
            'status',
         )

class JobSearchForm(forms.Form):
    q = forms.CharField(label=_("Search"), required=False, max_length=200,)
    categories = forms.ChoiceField(required=False)
    subcategories = forms.ChoiceField(required=False)
    
    def __init__(self, *args, **kwargs):
        super(JobSearchForm, self).__init__(*args, **kwargs)
        
        #setup categories        
        (categories, sub_categories) = Category.objects.get_for_model(Job)
        cat_length = len(categories)
        cat_choices = [('', 'Categories (%s)' % cat_length)]
        for category in categories:
            cat_choices.append((category.pk, category.name))

        query_string = args[0]
        category = query_string.get('categories', None)
        if category:
            sub_categories = Category.objects.get_for_model(Job, category)[1]
        subcat_length = len(sub_categories)
        subcat_choices = [('', 'Subcategories (%s)' % subcat_length)]
        for category in sub_categories:
            subcat_choices.append((category.pk, category.name))

        self.fields['categories'].choices = cat_choices
        self.fields['subcategories'].choices = subcat_choices



#import time
#import hashlib
import urllib
from django.conf import settings
#from django.http import Http404
from django.core.urlresolvers import reverse
from forms import PayflowLinkPaymentForm
from tendenci.core.payments.models import Payment
from tendenci.core.payments.utils import payment_processing_object_updates
from tendenci.core.event_logs.models import EventLog
from tendenci.apps.notifications.utils import send_notifications
from tendenci.core.payments.utils import log_payment, send_payment_notice
from django.utils.encoding import smart_str
#from tendenci.core.site_settings.utils import get_setting


def prepare_payflowlink_form(request, payment):
    amount = "%.2f" % payment.amount
    #payment.description = urllib.quote(smart_str(payment.description))
    params = {
              'login':settings.PAYPAL_MERCHANT_LOGIN,
              'partner': settings.PAYFLOWLINK_PARTNER,
              'amount':amount,
              'type': 'S',
              'showconfirm': 'True',
              'disablereceipt': 'False',
              'custid': payment.id,
              'name':'%s %s' % (payment.first_name, payment.last_name),
              'email': payment.email,
              'address': '%s %s' % (payment.address, payment.address2),
              'city': payment.city,
              'state': payment.state,
              'country': payment.country,
              'zip': payment.zip,
              'fax':payment.fax,
              'phone':payment.phone,
              'nametoship': '%s %s' % (payment.ship_to_first_name, payment.ship_to_last_name),
              'addresstoship': payment.ship_to_address,
              'citytoship': payment.ship_to_city,
              'statetoship': payment.ship_to_state,
              'ziptoship': payment.ship_to_zip,
              'countrytoship': payment.ship_to_country,
              'comment1': smart_str(payment.description).replace('#', ''),
              'comment2': '%s %s' % (payment.first_name, payment.last_name), 
        
        }
    form = PayflowLinkPaymentForm(initial=params)
    
    return form

def payflowlink_thankyou_processing(request, response_d, **kwargs):
    from django.shortcuts import get_object_or_404
    response_d = dict(map(lambda x: (x[0].lower(), x[1]), response_d.items()))

    paymentid = response_d.get('custid', 0)
    try:
        paymentid = int(paymentid)
    except:
        paymentid = 0
    payment = get_object_or_404(Payment, pk=paymentid)
    processed = False
    
    if payment.invoice.balance > 0:     # if balance==0, it means already processed
        payment_update_payflowlink(request, response_d, payment)
        payment_processing_object_updates(request, payment)
        processed = True
        
        # log an event
        log_payment(request, payment)
        
        # send payment recipients notification
        send_payment_notice(request, payment) 
        
    return payment, processed
        
def payment_update_payflowlink(request, response_d, payment, **kwargs):
    name = response_d.get('name', '')
    if name:
        name_list = name.split(' ')
        if len(name_list) >= 2:
            payment.first_name = name_list[0]
            payment.last_name = ' '.join(name_list[1:])
    payment.address = response_d.get('address', '')
    payment.city = response_d.get('city', '')
    payment.state = response_d.get('state', '')
    payment.country = response_d.get('country', '')
    payment.phone = response_d.get('phone', '')
    sname = response_d.get('nametoship', '')
    if sname:
        name_list = sname.split(' ')
        if len(name_list) >= 2:
            payment.ship_to_first_name = name_list[0]
            payment.ship_to_last_name = ' '.join(name_list[1:])
    payment.ship_to_address = response_d.get('addresstoship', '')
    payment.ship_to_city = response_d.get('citytoship', '')
    payment.ship_to_state = response_d.get('statetoship', '')
    payment.ship_to_zip = response_d.get('ziptoship', '')
    payment.ship_to_country = response_d.get('countrytoship', '')
    
    result = response_d.get('result', '')
    respmsg = (response_d.get('respmsg', '')).lower()
    payment.response_reason_text = respmsg
    
    if result=='0' and  respmsg == 'approved':
        payment.response_code = '1'
        payment.response_subcode = '1'
        payment.response_reason_code = '1'
        payment.status_detail = 'approved'
        # http://www.firstdata.com/downloads/marketing-merchant/fd_globalgatewayconnect_usermanualnorthamerica.pdf
        payment.trans_id = response_d.get('pnref', '')
    else:
        payment.response_code = 0
        payment.response_reason_code = 0
    
    
    if payment.is_approved:
        payment.mark_as_paid()
        payment.save()
        payment.invoice.make_payment(request.user, payment.amount)
    else:
        if not payment.status_detail:
            payment.status_detail = 'not approved'
        payment.save()
            
    
    
    



from django.template import Library, Node, TemplateSyntaxError, Variable, VariableDoesNotExist
from tendenci.core.files.models import File
from tendenci.core.base.template_tags import ListNode, parse_tag_kwargs

register = Library()


@register.inclusion_tag("files/options.html", takes_context=True)
def file_options(context, user, file):
    context.update({
        "opt_object": file,
        "user": user
    })
    return context


@register.inclusion_tag("files/nav.html", takes_context=True)
def file_nav(context, user, file=None):
    context.update({
        "nav_object": file,
        "user": user
    })
    return context


@register.inclusion_tag("files/search-form.html", takes_context=True)
def file_search(context):
    return context


@register.inclusion_tag('files/reports/most-viewed-result.html', takes_context=True)
def most_viewed_result(context):
    event_log = context['event_log']
    context['file'] = File.objects.get(pk=event_log['object_id'])
    return context


class FilesForModelNode(Node):

    def __init__(self, context_var, *args, **kwargs):
        self.kwargs = kwargs
        self.context_var = context_var

    def render(self, context):
        instance = self.kwargs['instance']

        try:
            instance = Variable(instance).resolve(context)
        except VariableDoesNotExist:
            return ''

        files = File.objects.get_for_model(instance)

        context[self.context_var] = files
        return ''


@register.tag
def files_for_model(parser, token):
    """
    Pull a list of :model:`File` objects based on another model.

    Example::

        {% files_for_model speaker as speaker_files %}
        {% for file in speaker_files %}
            {{ file.file }}
        {% endfor %}
    """
    args, kwargs = [], {}
    bits = token.split_contents()
    context_var = bits[3]
    kwargs['instance'] = bits[1]

    if len(bits) < 3:
        message = "'%s' tag requires more than 3" % bits[0]
        raise TemplateSyntaxError(message)

    if bits[2] != "as":
        message = "'%s' second argument must be 'as" % bits[0]
        raise TemplateSyntaxError(message)

    return FilesForModelNode(context_var, *args, **kwargs)


@register.filter
def size(file, size):
    """
    size examples:
        size = '100x200'
        size = '100'
        size = '100x'
        size = 'x100'
        size = '100x200/constrain'
    """
    from django.core.urlresolvers import reverse

    if not isinstance(file, File):
        return u''

    options = u''

    if '/' in size:
        size, options = size.split('/')

    kwargs = {
        'id': unicode(file.pk),
        'size': size,
    }

    if 'constrain' in options:
        kwargs['constrain'] = 'constrain'

    return reverse('file', kwargs=kwargs)


class ListFilesNode(ListNode):
    model = File
    perms = 'files.view_file'


@register.tag
def list_files(parser, token):
    """
    Used to pull a list of :model:`files.File` items.

    Usage::

        {% list_files as [varname] [options] %}

    Be sure the [varname] has a specific name like ``files_sidebar`` or
    ``files_list``. Options can be used as [option]=[value]. Wrap text values
    in quotes like ``tags="cool"``. Options include:

        ``limit``
           The number of items that are shown. **Default: 3**
        ``order``
           The order of the items. **Default: Newest Added**
        ``user``
           Specify a user to only show public items to all. **Default: Viewing user**
        ``query``
           The text to search for items. Will not affect order.
        ``tags``
           The tags required on items to be included.
        ``group``
           The group id associated with items to be included.
        ``random``
           Use this with a value of true to randomize the items included.

    Example::

        {% list_files as files_list limit=5 tags="cool" %}
        {% for file in files_list %}
            {{ file.name }}
        {% endfor %}
    """
    args, kwargs = [], {}
    bits = token.split_contents()
    context_var = bits[2]

    if len(bits) < 3:
        message = "'%s' tag requires more than 3" % bits[0]
        raise TemplateSyntaxError(message)

    if bits[1] != "as":
        message = "'%s' second argument must be 'as" % bits[0]
        raise TemplateSyntaxError(message)

    kwargs = parse_tag_kwargs(bits)

    if 'order' not in kwargs:
        kwargs['order'] = '-create_dt'

    return ListFilesNode(context_var, *args, **kwargs)



from haystack import indexes
from haystack import site

from django.utils.html import strip_tags, strip_entities

from tendenci.addons.resumes.models import Resume
from tendenci.core.perms.indexes import TendenciBaseSearchIndex


class ResumeIndex(TendenciBaseSearchIndex):
    title = indexes.CharField(model_attr='title')
    description = indexes.CharField(model_attr='description')
    activation_dt = indexes.DateTimeField(model_attr='activation_dt', null=True)
    expiration_dt = indexes.DateTimeField(model_attr='expiration_dt', null=True)
    syndicate = indexes.BooleanField(model_attr='syndicate')

    def prepare_description(self, obj):
        description = obj.description
        description = strip_tags(description)
        description = strip_entities(description)
        return description

site.register(Resume, ResumeIndex)



import os.path
import subprocess
from django.contrib.auth.decorators import login_required
from django.shortcuts import render_to_response, get_object_or_404, redirect
from django.template import RequestContext
from django.http import HttpResponseRedirect, HttpResponse 
from django.core.urlresolvers import reverse
from django.conf import settings
from django.contrib import messages
from django import forms
from django.db import models
from tendenci.apps.wp_importer.forms import BlogImportForm
from tendenci.apps.wp_importer.models import BlogImport
from tendenci.apps.wp_importer.tasks import WPImportTask
from tendenci.core.base.http import MissingApp
from django.contrib.auth.models import User
from django.contrib import messages
from django.utils.translation import ugettext as _
from parse_uri import ParseUri
from djcelery.models import TaskMeta

from tendenci.core.base.http import Http403
from tendenci.core.perms.utils import has_perm, update_perms_and_save
from tendenci.core.event_logs.models import EventLog
from tendenci.core.base.utils import send_email_notification

@login_required
def index(request, template_name="wp_importer/index.html"):
    if "tendenci.addons.articles" not in settings.INSTALLED_APPS:
        raise MissingApp('Oops, you must install Articles so that we can import your posts from WordPress!')
    if request.method == 'POST':
        form = BlogImportForm(request.POST, request.FILES)
        try:
            if form.is_valid() and request.FILES['blog'].name.endswith('xml') and request.FILES['blog'].size < 20*1024*1024 and request.user:
                upload = form.save(commit=False)
                upload.author = request.user
                upload = form.save()
                file_name = os.path.join(settings.MEDIA_ROOT, 'blogimport', request.FILES['blog'].name)
                
                result = WPImportTask.delay(file_name, request.user)
                #uncomment the next line if there is no celery server yet.
                #result.wait()
                subprocess.Popen(['python', 'manage.py', 'celeryd_detach'])

                return redirect("detail", result.task_id)
                
            elif not request.FILES['blog'].name.endswith('xml'):
                messages.add_message(
                    request,
                    messages.INFO,
                    _('Oops, only upload XML files!')
                )

            elif not request.FILES['blog'].size < 20*1024*1024:
                messages.add_message(
                    request,
                    messages.INFO,
                    _('Oops, only upload files smaller than 20 MB!')
                )
                
        except ValueError:
            messages.add_message(request, messages.INFO, 'Oops, please login before uploading a blog!')
            return redirect('auth_login')

    else:
        form = BlogImportForm()

    return render_to_response(template_name, {'form':form}, 
        context_instance=RequestContext(request))

@login_required
def detail(request, task_id, template_name="wp_importer/detail.html"):
    try:
        task = TaskMeta.objects.get(task_id=task_id)
    except TaskMeta.DoesNotExist:
        #tasks database entries are not created at once.
        #instead of raising 404 we'll assume that there will be one for
        #the id.
        task = None
    
    if task and task.status == "SUCCESS":
        messages.add_message(
            request,
            messages.INFO,
            _('Your blog has been imported!')
        )
        return render_to_response(template_name, {},
            context_instance=RequestContext(request))
    
    messages.add_message(
        request,
        messages.INFO,
        _("Your site import is being processed. You will receive an email at %s when the import is complete." % request.user.email)
    )
    
    return render_to_response(template_name, {},
        context_instance=RequestContext(request))



from haystack import indexes
from haystack import site

from tendenci.apps.redirects.models import Redirect


class RedirectIndex(indexes.RealTimeSearchIndex):
    text = indexes.CharField(document=True, use_template=True)
    from_url = indexes.CharField(model_attr='from_url')
    to_url = indexes.CharField(model_attr='to_url')
    http_status = indexes.IntegerField(model_attr='http_status')
    status = indexes.BooleanField(model_attr='status')
    uses_regex = indexes.BooleanField(model_attr='uses_regex')
    create_dt = indexes.DateTimeField(model_attr='create_dt')
    update_dt = indexes.DateTimeField(model_attr='update_dt')

    # PK: needed for exclude list_tags
    primary_key = indexes.CharField(model_attr='pk')

    def get_updated_field(self):
        return 'update_dt'

site.register(Redirect, RedirectIndex)



from tendenci.core.rss.feedsmanager import SubFeed
from tendenci.core.site_settings.utils import get_setting
from tendenci.core.perms.utils import PUBLIC_FILTER
from tendenci.core.sitemaps import TendenciSitemap

from tendenci.addons.resumes.models import Resume

class LatestEntriesFeed(SubFeed):
    title =  '%s Latest Resumes' % get_setting('site','global','sitedisplayname')
    link =  "/resumes/"
    description =  "Latest Resumes by %s" % get_setting('site','global','sitedisplayname')

    def items(self):
        items = Resume.objects.filter(**PUBLIC_FILTER).filter(syndicate=True).order_by('-create_dt')[:20]
        return items

    def item_title(self, item):
        return item.title

    def item_description(self, item):
        return item.description

    def item_pubdate(self, item):
        return item.create_dt

    def item_link(self, item):
        return item.get_absolute_url()

class ResumeSitemap(TendenciSitemap):
    """ Sitemap information for resumes """
    changefreq = "monthly"
    priority = 0.5

    def items(self):
        items = Resume.objects.filter(**PUBLIC_FILTER).order_by('-create_dt')
        return items

    def lastmod(self, obj):
        return obj.create_dt



from time import strptime, strftime
from south.modelsinspector import add_introspection_rules

from django.forms import fields, ValidationError
from django.db.models import CharField
from django.utils.translation import ugettext_lazy as _

from tendenci.core.base import forms
from tendenci.core.base.widgets import SplitDateTimeWidget, EmailVerificationWidget


# introspection rules for south migration for the slugfield
add_introspection_rules([], ['^tendenci\.core\.base\.fields\.SlugField'])


class SlugField(CharField):
    """
        New slug field that uses a different set of validations
        Straight copy from django with modifications
    """
    description = _("String (up to %(max_length)s)")
    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = kwargs.get('max_length', 100)
        # Set db_index=True unless it's been set manually.
        if 'db_index' not in kwargs:
            kwargs['db_index'] = True
        super(SlugField, self).__init__(*args, **kwargs)

    def get_internal_type(self):
        return "SlugField"

    def formfield(self, **kwargs):
        defaults = {'form_class': forms.SlugField}
        defaults.update(kwargs)
        return super(SlugField, self).formfield(**defaults)

class SplitDateTimeField(fields.MultiValueField):
    """
        Custom split date time widget
        Modified version of http://www.copiesofcopies.org/webl/?p=81
    """
    widget = SplitDateTimeWidget

    def __init__(self, *args, **kwargs):
        """
        Have to pass a list of field types to the constructor, else we
        won't get any data to our compress method.
        """
        all_fields = (
            fields.CharField(max_length=10),
            fields.CharField(max_length=8),
            )
        super(SplitDateTimeField, self).__init__(all_fields, *args, **kwargs)

    def compress(self, data_list):
        """
        Takes the values from the MultiWidget and passes them as a
        list to this function. This function needs to compress the
        list into a single object to save.
        """
        if data_list:
            if not (data_list[0] and data_list[1]):
                raise ValidationError("Field is missing data.")
            try:
                input_time = strptime("%s" % (data_list[1]), "%I:%M %p")
                datetime_string = "%s %s" % (data_list[0], strftime('%H:%M', input_time))
            except:
                raise ValidationError("Time Format is incorrect. Must be Hour:Minute AM|PM")
            return datetime_string
        return None


class EmailVerificationField(fields.MultiValueField):
    widget = EmailVerificationWidget

    def __init__(self, *args, **kwargs):
        """
        Have to pass a list of field types to the constructor, else we
        won't get any data to our compress method.
        """
        all_fields = (
            fields.EmailField(max_length=100),
            fields.EmailField(max_length=100, label=_("Verfiy Email Address")),
            )
        label = kwargs.pop('label', '') + ' (Enter twice to verify)'
        super(EmailVerificationField, self).__init__(all_fields, label=label, *args, **kwargs)

    def compress(self, data_list):
        """
        Takes the values from the MultiWidget and passes them as a
        list to this function. This function needs to compress the
        list into a single object to save.
        """
        if data_list:
            if not (data_list[0] and data_list[1]):
                raise ValidationError("Please enter the email twice to verify.")
            if data_list[0] != data_list[1]:
                raise ValidationError("Please enter similar email addresses.")
            email_data = "%s" % (data_list[0])
            return email_data
        return ''



from django.core.management.base import BaseCommand


class Command(BaseCommand):
    """
    Command to move data from previous table names (singular notification)

    example: python manage.py move_from_old_table
    """
    def handle(self, *args, **kwargs):
        from tendenci.apps.notifications import models

        # NoticeType
        try:
            for i in models.NoticeType.objects.raw('SELECT * FROM notification_noticetype'):
                i.save()
        except:
            pass

        # NoticeSetting
        try:
            for i in models.NoticeSetting.objects.raw('SELECT * FROM notification_noticesetting'):
                i.save()
        except:
            pass

        # Notice
        try:
            for i in models.Notice.objects.raw('SELECT * FROM notification_notice'):
                i.save()
        except:
            pass

        # NoticeQueueBatch
        try:
            for i in models.NoticeQueueBatch.objects.raw('SELECT * FROM notification_noticequeuebatch'):
                i.save()
        except:
            pass

        # NoticeEmail
        try:
            for i in models.NoticeEmail.objects.raw('SELECT * FROM notification_noticeemail'):
                i.save()
        except:
            pass

        # ObservedItem
        try:
            for i in models.ObservedItem.objects.raw('SELECT * FROM notification_observeditem'):
                i.save()
        except:
            pass

        # Update the auto increments on the ID fields
        models.NoticeType.objects.raw('SELECT setval(pg_get_serial_sequence(\'"notifications_noticetype"\',\'id\'), coalesce(max("id"), 1), max("id") IS NOT null) FROM "notifications_noticetype"')
        models.NoticeSetting.objects.raw('SELECT setval(pg_get_serial_sequence(\'"notifications_noticesetting"\',\'id\'), coalesce(max("id"), 1), max("id") IS NOT null) FROM "notifications_noticesetting"')
        models.Notice.objects.raw('SELECT setval(pg_get_serial_sequence(\'"notifications_notice"\',\'id\'), coalesce(max("id"), 1), max("id") IS NOT null) FROM "notifications_notice"')
        models.NoticeQueueBatch.objects.raw('SELECT setval(pg_get_serial_sequence(\'"notifications_noticequeuebatch"\',\'id\'), coalesce(max("id"), 1), max("id") IS NOT null) FROM "notifications_noticequeuebatch"')
        models.NoticeEmail.objects.raw('SELECT setval(pg_get_serial_sequence(\'"notifications_noticeemail"\',\'id\'), coalesce(max("id"), 1), max("id") IS NOT null) FROM "notifications_noticeemail"')
        models.ObservedItem.objects.raw('SELECT setval(pg_get_serial_sequence(\'"notifications_observeditem"\',\'id\'), coalesce(max("id"), 1), max("id") IS NOT null) FROM "notifications_observeditem"')



from os.path import join
from django.conf.urls.defaults import *
from django.conf import settings
from django.views.generic.simple import direct_to_template, redirect_to
from django.contrib import admin
from model_report import report

from tendenci.core.registry import autodiscover as reg_autodiscover
from tendenci.core.newsletters.views import NewsletterGeneratorView

# load the apps that are in Django Admin
admin.autodiscover()

# load the app_registry
reg_autodiscover()

# django model report
report.autodiscover()

# Admin Patterns
urlpatterns = patterns('',
    (r'^admin/doc/', include('django.contrib.admindocs.urls')),
    (r'^admin/', include(admin.site.urls)),
)

#report patterns
urlpatterns += patterns('',
    (r'^model-report/', include('model_report.urls')),
)

# Tendenci Patterns
urlpatterns += patterns('',
    url(r'^$', 'tendenci.core.base.views.homepage', name="home"),

    #Reports:
    (r'^metrics/', include('tendenci.apps.metrics.urls')),
    url(r'^users/reports/users-activity-top10/$', 'tendenci.apps.profiles.views.user_activity_report', name='reports-user-activity'),
    url(r'^users/reports/active-logins/$', 'tendenci.apps.profiles.views.user_access_report', name='reports-user-access'),
    url(r'^users/reports/not-in-groups/$', 'tendenci.apps.profiles.views.users_not_in_groups', name='reports-users-not-in-groups'),
    url(r'^users/reports/admin/$', 'tendenci.apps.profiles.views.admin_users_report', name='reports-admin-users'),
    url(r'^users/reports/users-added/$', 'tendenci.apps.user_groups.views.users_added_report', {'kind': 'added'}, name='reports-user-added'),
    url(r'^users/reports/contacts-referral/$', 'tendenci.apps.user_groups.views.users_added_report', {'kind': 'referral'}, name='reports-contacts-referral'),

    (r'^notifications/', include('tendenci.apps.notifications.urls')),
    (r'^base/', include('tendenci.core.base.urls')),
    (r'^tags/', include('tendenci.core.tags.urls')),
    (r'^avatar/', include('avatar.urls')),
    (r'^dashboard/', include('tendenci.apps.dashboard.urls')),
    (r'^categories/', include('tendenci.core.categories.urls')),
    (r'^memberships/', include('tendenci.addons.memberships.urls')),
    (r'^corporatememberships/', include('tendenci.addons.corporate_memberships.urls')),
    (r'^entities/', include('tendenci.apps.entities.urls')),
    (r'^pages/', include('tendenci.apps.pages.urls')),
    (r'^users/', include('tendenci.apps.profiles.urls')),
    (r'^photos/', include('tendenci.addons.photos.urls')),
    (r'^forms/', include('tendenci.apps.forms_builder.forms.urls')),
    (r'^events/', include('tendenci.addons.events.urls')),
    (r'^profiles/', include('tendenci.apps.profiles.urls')),
    (r'^groups/', include('tendenci.apps.user_groups.urls')),
    (r'^stories/', include('tendenci.apps.stories.urls')),
    (r'^invoices/', include('tendenci.apps.invoices.urls')),
    (r'^py/', include('tendenci.addons.make_payments.urls')),
    (r'^payments/', include('tendenci.core.payments.urls')),
    (r'^rp/', include('tendenci.addons.recurring_payments.urls')),
    (r'^accountings/', include('tendenci.apps.accountings.urls')),
    (r'^emails/', include('tendenci.core.emails.urls')),
    (r'^rss/', include('tendenci.core.rss.urls')),
    (r'^imports/', include('tendenci.core.imports.urls')),
    #(r'^donations/', include('donations.urls')),
    (r'^settings/', include('tendenci.core.site_settings.urls')),
    (r'^files/', include('tendenci.core.files.urls')),
    (r'^accounts/', include('tendenci.apps.accounts.urls')),
    (r'^search/', include('tendenci.apps.search.urls')),
    (r'^event-logs/', include('tendenci.core.event_logs.urls')),
    (r'^contributions/', include('tendenci.apps.contributions.urls')),
    (r'^theme-editor/', include('tendenci.apps.theme_editor.urls')),
    (r'^exports/', include('tendenci.core.exports.urls')),
    (r'^ics/', include('tendenci.addons.events.ics.urls')),
    (r'^boxes/', include('tendenci.apps.boxes.urls')),
    (r'^sitemap.xml', include('tendenci.core.sitemaps.urls')),
    (r'^404/', include('tendenci.core.handler404.urls')),

    (r'^subscribers/', include('tendenci.apps.subscribers.urls')),
    (r'^redirects/', include('tendenci.apps.redirects.urls')),
    (r'^resumes/', include('tendenci.addons.resumes.urls')),
    (r'^mobile/', include('tendenci.core.mobile.urls')),
    (r'^campaign_monitor/', include('tendenci.addons.campaign_monitor.urls')),
    (r'^wp_importer/', include('tendenci.apps.wp_importer.urls')),
    (r'^wp_exporter/', include('tendenci.apps.wp_exporter.urls')),
    (r'^discounts/', include('tendenci.apps.discounts.urls')),
    (r'^versions/', include('tendenci.core.versions.urls')),
    url(r'social_auth/', include('tendenci.addons.social_auth.urls')),
    url(r'navs/', include('tendenci.apps.navs.urls')),
    url(r'tendenci/', include('tendenci.addons.tendenci_guide.urls')),
    url(r'^api_tasty/', include('tendenci.core.api_tasty.urls')),

    # third party (inside environment)
    (r'^tinymce/', include('tinymce.urls')),
    (r'^captcha/', include('captcha.urls')),

    url(r'^sitemap/$', direct_to_template, {"template": "site_map.html", }, name="site_map"),
    url(r'^robots.txt', 'tendenci.core.base.views.robots_txt', name="robots"),

    # legacy redirects
    url(r'^login/$', redirect_to, {'url': '/accounts/login/'}),
    (r'^newsletter_generator/', NewsletterGeneratorView.as_view()),

    url(r'^', include('tendenci.apps.contacts.urls')),
    url(r'^', include('tendenci.addons.articles.urls')),
    url(r'^', include('tendenci.addons.jobs.urls')),
    url(r'^', include('tendenci.addons.directories.urls')),
    url(r'^', include('tendenci.addons.help_files.urls')),
    url(r'^', include('tendenci.addons.locations.urls')),
    url(r'^', include('tendenci.addons.news.urls')),
)

handler500 = 'tendenci.core.base.views.custom_error'


if hasattr(settings, 'USE_S3_STORAGE') and settings.USE_S3_STORAGE:
    urlpatterns += patterns('',
    # serve .less files - this is to resolve the cross domain issue for less js
    url(r'^(?P<path>.*)\.less$', 
        'tendenci.core.files.views.display_less',  name='less_file'),
    url(r'^static/(?P<path>.*)$',
            'tendenci.core.files.views.redirect_to_s3',
             {'file_type': 'static'},
                name='redirect_to_s3'),
    # this is basically for those images with relative urls in the theme .css files.
    url(r'^themes/(?P<path>.*)$',
            'tendenci.core.files.views.redirect_to_s3',
            {'file_type': 'themes'},
                name='redirect_to_s3'),
)

# serve static files
if settings.DEBUG:
    if not (hasattr(settings, 'USE_S3_STORAGE') and settings.USE_S3_STORAGE):
        urlpatterns += patterns('',
            (r'^static/(?P<path>.*)$',
                'django.views.static.serve',
                {'document_root': join(settings.TENDENCI_ROOT, 'static')}),

            (r'^plugin-media/(?P<plugin>[^/]+)/(?P<path>.*)$',
                'tendenci.core.base.views.plugin_static_serve'),
        )
        urlpatterns += patterns('',
            (r'^themes/(?P<path>.*)$',
                'django.views.static.serve',
                {'document_root': settings.THEMES_DIR, 'show_indexes': True}),
        )

# Favicon url to prevent 404 from some browsers.
urlpatterns += patterns('',
    (r'^(?P<path>favicon\.ico)$',
        'django.views.static.serve',
        {'document_root': join(settings.STATIC_ROOT, 'images'), 'show_indexes': True}),
)

# template tag testing environment
if settings.DEBUG:
    urlpatterns += patterns('',
        url(r'^tag-test/$', direct_to_template, {"template": "tag_test.html", }, name="tag_test"),
    )

# Local url patterns for development
try:
    from local_urls import extra_patterns
    urlpatterns += extra_patterns
except ImportError:
    pass

#PLUGINS:
from tendenci.core.registry.utils import get_url_patterns
urlpatterns += get_url_patterns()

urlpatterns += patterns('', url(r'^en/$', redirect_to, {'url': '/accounts/login/'}),)

# tack on the pages pattern at the very end so let custom and software patterns
# happen first
pattern_pages = patterns('',
    # page view
    url(r'^(?P<slug>[\w\-\/]+)/$', 'tendenci.apps.pages.views.index', name="page"),  
)
urlpatterns += pattern_pages



from django.core.cache import cache
from django.conf import settings

non_model_event_logs = {
    'rss': {
        'feed_base':('630000', '00FF00'), # rss feed base
        'global_feed':('630500', '00FF00'), # global rss feed
        'article_feed':('630543', '00FF33'), # articles rss feed
        'news_feed':('630530', '00FFCC'), # news rss feed
        'pages_feed':('630560', '33FF99'), # pages rss feed
        'jobs_feed':('630525', '00FF99'), # jobs rss feed
    },
    'homepage':{
        'view':('100000', '7F0000'), # base app
    }
}

def generate_colors():
    """Create the event id to color dict so we won't
    have to iterate over the apps in the event registry
    for every event id.
    """
    from tendenci.core.registry import site
    d = {}
    apps = site.get_registered_apps().all_apps
    for app in apps:
        if 'event_logs' in app:
            for model in app['event_logs'].keys():
                for event in app['event_logs'][model].keys():
                    log_id = app['event_logs'][model][event][0]
                    color = app['event_logs'][model][event][1]
                    d[log_id] = color
    return d
    
def generate_base_colors():
    """Crete the event id to color dict for event logs that
    are not associated with any model or registry.
    """
    d = {}
    for model in non_model_event_logs.keys():
        for event in non_model_event_logs[model].keys():
            log_id = non_model_event_logs[model][event][0]
            color = non_model_event_logs[model][event][1]
            d[log_id] = color
    return d

def cache_colors(colors):
    keys = [settings.CACHE_PRE_KEY, 'event_log_colors']
    key = '.'.join(keys)
    is_set = cache.add(key, colors)
    if not is_set:
        cache.set(key, colors)

def get_color(event_id):
    """Gets the hex color of an event log based on the event id
    get_color('id')
    """
    keys = [settings.CACHE_PRE_KEY, 'event_log_colors']
    key = '.'.join(keys)
    colors = cache.get(key)
    if not colors:
        colors = generate_colors()
        colors.update(generate_base_colors())
        cache_colors(colors)
    
    if event_id in colors:
        return colors[event_id]
    else:
        return '333333'
    
    return ''



from datetime import datetime
from haystack import indexes
from haystack import site

from django.utils.html import strip_tags, strip_entities

from tendenci.addons.news.models import News
from tendenci.core.categories.models import Category
from tendenci.core.perms.indexes import TendenciBaseSearchIndex


class NewsIndex(TendenciBaseSearchIndex):
    headline = indexes.CharField(model_attr='headline')
    body = indexes.CharField(model_attr='body')
    release_dt = indexes.DateTimeField(model_attr='release_dt', null=True)
    syndicate = indexes.BooleanField(model_attr='syndicate')
    
    # categories
    category = indexes.CharField()
    sub_category = indexes.CharField()

    # RSS fields
    can_syndicate = indexes.BooleanField()
    order = indexes.DateTimeField()
    
    def prepare_body(self, obj):
        body = obj.body
        body = strip_tags(body)
        body = strip_entities(body)
        return body
    
    def prepare_category(self, obj):
        category = Category.objects.get_for_object(obj, 'category')
        if category:
            return category.name
        return ''

    def prepare_sub_category(self, obj):
        category = Category.objects.get_for_object(obj, 'sub_category')
        if category:
            return category.name
        return ''

    def prepare_can_syndicate(self, obj):
        return obj.allow_anonymous_view and obj.syndicate \
                and obj.status == 1  and obj.status_detail == 'active' \
                and obj.release_dt <= datetime.now()

    def prepare_order(self, obj):
        return obj.release_dt

site.register(News, NewsIndex)



import commands
from django.conf import settings
from django.core.management.base import BaseCommand
from datetime import datetime
from django.core.management import call_command

class Command(BaseCommand):
    """
    example: python manage.py expire_resumes
    """
    def handle(self, *args, **kwargs):
        from tendenci.addons.resumes.models import Resume
        for resume in Resume.objects.filter(status_detail='active'):
            if resume.expiration_dt < datetime.now():
                resume.status_detail = 'expired'
                resume.save()
        call_command('update_index', *["resumes"])




from django.contrib.auth.decorators import login_required
from django.shortcuts import get_object_or_404, redirect
from django.template import RequestContext
from django.http import HttpResponseRedirect, Http404
from django.core.urlresolvers import reverse

from django.contrib import messages
from django.contrib.contenttypes.models import ContentType
from django.db.models import Q

from tendenci.core.base.http import Http403
from tendenci.core.base.utils import template_exists
from tendenci.core.base.views import file_display
from tendenci.core.site_settings.utils import get_setting
from tendenci.core.event_logs.models import EventLog
from tendenci.core.meta.models import Meta as MetaTags
from tendenci.core.meta.forms import MetaForm
from tendenci.core.versions.models import Version
from tendenci.core.perms.utils import (update_perms_and_save,
                                       get_notice_recipients,
                                       has_perm,
                                       has_view_perm,
                                       get_query_filters)
from tendenci.core.categories.forms import CategoryForm
from tendenci.core.categories.models import Category
from tendenci.core.theme.shortcuts import themed_response as render_to_response
from tendenci.core.exports.utils import run_export_task
from tendenci.apps.notifications import models as notification

from tendenci.apps.pages.models import Page, HeaderImage
from tendenci.apps.pages.forms import PageForm


def index(request, slug=None, id=None, hash=None, 
          template_name="pages/view.html"):
    """
    Return page object, either as an archive, active, or version.
    """

    if not slug and not id and not hash:
        return HttpResponseRedirect(reverse('page.search'))

    if hash:
        version = get_object_or_404(Version, hash=hash)
        current_page = get_object_or_404(Page, pk=version.object_id)
        page = version.get_version_object()
        messages.add_message(request, messages.WARNING,
         'You are viewing a previous version of this article. View the ' + \
         '<a href="%s">Current Version</a>.' % current_page.get_absolute_url())
    elif id:
        page = get_object_or_404(Page, pk=id)
        if page.status_detail != 'active':
            if not request.user.is_authenticated():
                pages = Page.objects.filter(
                    slug=page.slug, status_detail='active'
                ).order_by('-pk')
                if not pages:
                    pages = Page.objects.filter(slug=slug).order_by('-pk')
                if not pages:
                    raise Http404
                return HttpResponseRedirect(reverse('page', args=[page.slug]))

    else:
        try:
            page = get_object_or_404(Page, slug=slug)
        except Page.MultipleObjectsReturned:
            pages = Page.objects.filter(
                slug=slug, status_detail='active'
            ).order_by('-pk')
            if not pages:
                pages = Page.objects.filter(slug=slug).order_by('-pk')
            if not pages:
                raise Http404

            page = pages[0]

    if not has_view_perm(request.user, 'pages.view_page', page):
        raise Http403

    if not page.template or not template_exists(page.template):
        page.template = "pages/base.html"

    EventLog.objects.log(instance=page)

    return render_to_response(template_name, {'page': page},
        context_instance=RequestContext(request))


def search(request, template_name="pages/search.html"):
    """
    Search pages.
    """
    query = request.GET.get('q', None)

    if get_setting('site', 'global', 'searchindex') and query:
        pages = Page.objects.search(query, user=request.user)
    else:
        filters = get_query_filters(request.user, 'pages.view_page')
        pages = Page.objects.filter(filters).distinct()
        if query:
            pages = pages.filter(Q(title__icontains=query) \
                             | Q(content__icontains=query) \
                              | Q(slug__icontains=query))
            pages = pages.exclude(status_detail='archive')

    pages = pages.order_by('-create_dt')

    EventLog.objects.log()

    return render_to_response(template_name, {'pages': pages},
        context_instance=RequestContext(request))


def print_view(request, slug, template_name="pages/print-view.html"):
    page = get_object_or_404(Page, slug=slug, status_detail='active')

    if not has_perm(request.user, 'pages.view_page', page):
        raise Http403

    EventLog.objects.log(instance=page)

    return render_to_response(template_name, {'page': page},
        context_instance=RequestContext(request))

@login_required
def edit(request, id, form_class=PageForm,
         meta_form_class=MetaForm,
         category_form_class=CategoryForm, template_name="pages/edit.html"):
    page = get_object_or_404(Page, pk=id)

    if not has_perm(request.user, 'pages.change_page', page):
        raise Http403

    content_type = get_object_or_404(ContentType, app_label='pages',
                                     model='page')

    #setup categories
    category = Category.objects.get_for_object(page, 'category')
    sub_category = Category.objects.get_for_object(page, 'sub_category')

    initial_category_form_data = {
        'app_label': 'pages',
        'model': 'page',
        'pk': page.pk,
        'category': getattr(category, 'name', '0'),
        'sub_category': getattr(sub_category, 'name', '0')
    }

    if request.method == "POST":
        form = form_class(request.POST, request.FILES,
                          instance=page,
                          user=request.user)
        metaform = meta_form_class(request.POST,
                                   instance=page.meta,
                                   prefix='meta')
        categoryform = category_form_class(content_type,
                                           request.POST,
                                           initial=initial_category_form_data,
                                           prefix='category')
        if form.is_valid() and metaform.is_valid() and categoryform.is_valid():
            page = form.save()

            # handle header image
            f = form.cleaned_data['header_image']
            if f:
                header = HeaderImage()
                header.content_type = ContentType.objects.get_for_model(Page)
                header.object_id = page.id
                header.creator = request.user
                header.creator_username = request.user.username
                header.owner = request.user
                header.owner_username = request.user.username
                filename = "%s-%s" % (page.slug, f.name)
                f.file.seek(0)
                header.file.save(filename, f)
                page.header_image = header

            #save meta
            meta = metaform.save()
            page.meta = meta

            ## update the category of the article
            category_removed = False
            category = categoryform.cleaned_data['category']
            if category != '0':
                Category.objects.update(page, category, 'category')
            else:  # remove
                category_removed = True
                Category.objects.remove(page, 'category')
                Category.objects.remove(page, 'sub_category')

            if not category_removed:
                # update the sub category of the article
                sub_category = categoryform.cleaned_data['sub_category']
                if sub_category != '0':
                    Category.objects.update(page, sub_category, 'sub_category')
                else:  # remove
                    Category.objects.remove(page, 'sub_category')

            # update all permissions
            page = update_perms_and_save(request, form, page)

            messages.add_message(request, messages.SUCCESS,
                                 'Successfully updated %s' % page)

            if not request.user.profile.is_superuser:
                # send notification to administrators
                recipients = get_notice_recipients('module', 'pages',
                                                   'pagerecipients')
                if recipients:
                    if notification:
                        extra_context = {
                            'object': page,
                            'request': request,
                        }
                        notification.send_emails(recipients,
                                                 'page_edited',
                                                 extra_context)

            return HttpResponseRedirect(reverse('page', args=[page.slug]))
    else:
        form = form_class(instance=page, user=request.user)
        metaform = meta_form_class(instance=page.meta, prefix='meta')
        categoryform = category_form_class(content_type,
                                           initial=initial_category_form_data,
                                           prefix='category')

    return render_to_response(template_name,
        {
            'page': page,
            'form': form,
            'metaform': metaform,
            'categoryform': categoryform,
        },
        context_instance=RequestContext(request))


@login_required
def edit_meta(request, id, form_class=MetaForm, template_name="pages/edit-meta.html"):
    """
    Return page that allows you to edit meta-html information.
    """

    # check permission
    page = get_object_or_404(Page, pk=id)
    if not has_perm(request.user, 'pages.change_page', page):
        raise Http403

    defaults = {
        'title': page.get_title(),
        'description': page.get_description(),
        'keywords': page.get_keywords(),
        'canonical_url': page.get_canonical_url(),
    }
    page.meta = MetaTags(**defaults)

    if request.method == "POST":
        form = form_class(request.POST, instance=page.meta)
        if form.is_valid():
            page.meta = form.save()  # save meta
            page.save()  # save relationship

            messages.add_message(request, messages.SUCCESS,
                                 'Successfully updated meta for %s' % page)

            return HttpResponseRedirect(reverse('page', args=[page.slug]))
    else:
        form = form_class(instance=page.meta)

    return render_to_response(template_name, {'page': page, 'form': form},
        context_instance=RequestContext(request))

@login_required
def preview(request, id=None, form_class=PageForm, meta_form_class=MetaForm,
        category_form_class=CategoryForm, template="pages/preview.html"):

    content_type = get_object_or_404(ContentType,
                                     app_label='pages',
                                     model='page')
    page = None
    if id:
        page = get_object_or_404(Page, pk=id)

    if request.method == "POST":
        if page:
            form = form_class(request.POST, request.FILES, instance=page, user=request.user)
        else:
            form = form_class(request.POST, request.FILES, user=request.user)
        metaform = meta_form_class(request.POST, prefix='meta')
        categoryform = category_form_class(content_type,
                                           request.POST,
                                           prefix='category')
        if form.is_valid():
           page = form.save(commit=False)

           edit_button = False
           if request.POST['preview_for'] == 'edit':
               edit_button = True 
           
           f = form.cleaned_data['header_image']
           if f:
               header = HeaderImage()
               header.content_type = ContentType.objects.get_for_model(Page)
               header.object_id = page.id
               header.creator = request.user
               header.creator_username = request.user.username
               header.owner = request.user
               header.owner_username = request.user.username
               filename = "%s-%s" % (page.slug, f.name)
               f.file.seek(0)
               header.file.save(filename, f, save=False)
               page.header_image = header
           
           return render_to_response(template, {'page': page,
                                                'form': form,
                                                'metaform': metaform,
                                                'categoryform': categoryform,
                                                'edit_button': edit_button},
               context_instance=RequestContext(request))
        
    return HttpResponseRedirect(reverse('page.search'))

@login_required
def add(request, form_class=PageForm, meta_form_class=MetaForm,
        category_form_class=CategoryForm,
        template_name="pages/add.html"):

    if not has_perm(request.user, 'pages.add_page'):
        raise Http403

    content_type = get_object_or_404(ContentType,
                                     app_label='pages',
                                     model='page')

    if request.method == "POST":
        form = form_class(request.POST, request.FILES, user=request.user)
        metaform = meta_form_class(request.POST, prefix='meta')
        categoryform = category_form_class(content_type,
                                           request.POST,
                                           prefix='category')
        if form.is_valid() and metaform.is_valid() and categoryform.is_valid():
            page = form.save()

            # handle header image
            f = form.cleaned_data['header_image']
            if f:
                header = HeaderImage()
                header.content_type = ContentType.objects.get_for_model(Page)
                header.object_id = page.id
                header.creator = request.user
                header.creator_username = request.user.username
                header.owner = request.user
                header.owner_username = request.user.username
                filename = "%s-%s" % (page.slug, f.name)
                f.file.seek(0)
                header.file.save(filename, f)
                page.header_image = header

            #save meta
            meta = metaform.save()
            page.meta = meta

            #setup categories
            category = Category.objects.get_for_object(page, 'category')
            sub_category = Category.objects.get_for_object(page,
                                                           'sub_category')

            ## update the category of the article
            category_removed = False
            category = categoryform.cleaned_data['category']
            if category != '0':
                Category.objects.update(page, category, 'category')
            else:  # remove
                category_removed = True
                Category.objects.remove(page, 'category')
                Category.objects.remove(page, 'sub_category')

            if not category_removed:
                # update the sub category of the article
                sub_category = categoryform.cleaned_data['sub_category']
                if sub_category != '0':

                    Category.objects.update(page, sub_category, 'sub_category')
                else:  # remove
                    Category.objects.remove(page, 'sub_category')

            # add all permissions
            page = update_perms_and_save(request, form, page)

            messages.add_message(request, messages.SUCCESS,
                                 'Successfully added %s' % page)

            if not request.user.profile.is_superuser:
                # send notification to administrators
                recipients = get_notice_recipients('module',
                                                   'pages',
                                                    'pagerecipients')
                if recipients:
                    if notification:
                        extra_context = {
                            'object': page,
                            'request': request,
                        }
                        notification.send_emails(recipients, 'page_added', extra_context)
            if page.status and page.status_detail == 'active':
                return HttpResponseRedirect(reverse('page', args=[page.slug]))
            return HttpResponseRedirect(reverse('page.search'))
    else:
        initial_category_form_data = {
            'app_label': 'pages',
            'model': 'page',
            'pk': 0,
        }
        form = form_class(user=request.user)
        metaform = meta_form_class(prefix='meta')
        categoryform = category_form_class(content_type,
                                           initial=initial_category_form_data,
                                           prefix='category')
    return render_to_response(template_name,
            {
                'form': form,
                'metaform': metaform,
                'categoryform': categoryform,
            },
            context_instance=RequestContext(request))


@login_required
def delete(request, id, template_name="pages/delete.html"):
    page = get_object_or_404(Page, pk=id)

    if not has_perm(request.user, 'pages.delete_page'):
        raise Http403

    if request.method == "POST":
        EventLog.objects.log(instance=page)
        messages.add_message(request, messages.SUCCESS,
                             'Successfully deleted %s' % page)

        # send notification to administrators
        recipients = get_notice_recipients('module', 'pages', 'pagerecipients')
        if recipients:
            if notification:
                extra_context = {
                    'object': page,
                    'request': request,
                }
                notification.send_emails(recipients, 'page_deleted', extra_context)

        # Soft delete
        page.status = False
        page.status_detail = 'inactive'
        page.save()
        return HttpResponseRedirect(reverse('page.search'))

    return render_to_response(template_name, {'page': page},
        context_instance=RequestContext(request))


def display_header_image(request, id):
    page = get_object_or_404(Page, pk=id)

    if not has_view_perm(request.user,
                        '[pages.view_page',
                        page):
        raise Http403

    return file_display(request, page.header_image.file.name)


@login_required
def export(request, template_name="pages/export.html"):
    """Export Pages"""

    if not request.user.is_superuser:
        raise Http403

    if request.method == 'POST':

        fields = [
            'guid',
            'title',
            'slug',
            'header_image',
            'content',
            'view_contact_form',
            'design_notes',
            'syndicate',
            'template',
            'tags',
            'entity',
            'meta',
            'categories',
        ]

        export_id = run_export_task('pages', 'page', fields)
        return redirect('export.status', export_id)

    return render_to_response(template_name, {
    }, context_instance=RequestContext(request))



from datetime import datetime
import time
from decimal import Decimal
from django.template.loader import render_to_string
from django.template import TemplateDoesNotExist
from django.conf import settings
from django.contrib.auth.models import User
from django.core.urlresolvers import reverse
from dateutil.relativedelta import relativedelta
from tendenci.core.emails.models import Email
from tendenci.core.site_settings.utils import get_setting
from tendenci.apps.profiles.models import Profile
from tendenci.addons.recurring_payments.models import (RecurringPayment, 
                                       PaymentProfile, 
                                       RecurringPaymentInvoice,
                                       PaymentTransaction)
from tendenci.addons.recurring_payments.authnet.cim import CIMCustomerProfile, CIMHostedProfilePage
from tendenci.addons.recurring_payments.authnet.utils import get_token
from tendenci.addons.recurring_payments.authnet.utils import payment_update_from_response
from tendenci.core.payments.models import Payment

UNSUCCESSFUL_TRANS_CODE = ['E00027']
        
  
class RecurringPaymentEmailNotices(object):
    def __init__(self):
        self.site_display_name = get_setting('site', 'global', 'sitedisplayname')
        self.site_contact_name = get_setting('site', 'global', 'sitecontactname')
        self.site_contact_email = get_setting('site', 'global', 'sitecontactemail')
        self.reply_to_email = get_setting('module', 'payments', 'paymentrecipients')
        if not self.reply_to_email:
            self.reply_to_email = self.site_contact_email
        self.site_url = get_setting('site', 'global', 'siteurl')
    
        self.email = Email()
        self.email.sender = get_setting('site', 'global', 'siteemailnoreplyaddress')
        self.email.sender_display = self.site_display_name
        self.email.reply_to = self.reply_to_email
        
        self.admin_emails = self.get_admin_emails() 

    def get_admin_emails(self):
        payment_admins = get_setting('module', 'payments', 'paymentrecipients')
        if payment_admins:
            payment_admins = payment_admins.split(',')
            admin_emails = payment_admins
        else:
            admin_emails = (get_setting('site', 'global', 'admincontactemail')).split(',')
            
        if admin_emails:
            admin_emails = ','.join(admin_emails)
            
        return admin_emails
    
    def get_script_support_emails(self):
        admins = getattr(settings, 'ADMINS', None) 
        if admins:
            recipients_list = [admin[1] for admin in admins]
            return ','.join(recipients_list)
        
        return None
        
    def email_script_support_transaction_error(self, payment_transaction):
        """if there is an error other than transaction not being approved, notify us.
        """
        self.email.recipient = self.get_script_support_emails()
        if self.email.recipient:
            template_name = "recurring_payments/email_script_support_transaction.html"
            try:
                email_content = render_to_string(template_name, 
                                               {'pt':payment_transaction,
                                                'site_display_name': self.site_display_name,
                                                'site_url': self.site_url
                                                })
                self.email.body = email_content
                self.email.content_type = "html"
                self.email.priority = 1
                self.email.subject = 'Recurring payment transaction error on %s' % ( 
                                                                            self.site_display_name)
                
                self.email.send()
            except TemplateDoesNotExist:
                pass
        
    def email_admins_transaction_result(self, payment_transaction, success=True):
        """Send admins the result after the transaction is processed.
        """
        self.email.recipient = self.admin_emails
        if self.email.recipient:
            template_name = "recurring_payments/email_admins_transaction.html"
            user_in_texas = False
            if payment_transaction.payment.state:
                if payment_transaction.payment.state.lower() in ['texas', 'tx']:
                    user_in_texas = True
            try:
                email_content = render_to_string(template_name, 
                                               {'pt':payment_transaction,
                                                'site_display_name': self.site_display_name,
                                                'site_url': self.site_url,
                                                'user_in_texas': user_in_texas
                                                })
                self.email.body = email_content
                self.email.content_type = "html"
                if not success:
                    self.email.subject = 'Recurring payment transaction failed on %s' % ( 
                                                                            self.site_display_name)
                    self.email.priority = 1
                else:
                    self.email.subject = 'Recurring payment transaction processed on %s' % ( 
                                                                                self.site_display_name)
                
                self.email.send()
            except TemplateDoesNotExist:
                pass

    def email_customer_transaction_result(self, payment_transaction):
        """Send customer an email after the transaction is processed.
        """
        self.email.recipient = payment_transaction.recurring_payment.user.email
        if self.email.recipient:
            template_name = "recurring_payments/email_customer_transaction.html"
            try:
                email_content = render_to_string(template_name, 
                                               {'pt':payment_transaction,
                                                'site_display_name': self.site_display_name,
                                                'site_url': self.site_url
                                                })
                self.email.body = email_content
                self.email.content_type = "html"
                if payment_transaction.status:
                    self.email.subject = 'Payment received ' 
                else:
                    self.email.subject = 'Payment failed ' 
                self.email.subject = "%s for %s " % (self.email.subject, 
                                                   payment_transaction.recurring_payment.description)
                
                self.email.send()
            except TemplateDoesNotExist:
                pass

    def email_admins_no_payment_profile(self, recurring_payment):
        """Notify admin that payment method hasn't been setup yet for this recurring payment entry.
        """
        self.email.recipient = self.admin_emails
        if self.email.recipient:
            template_name = "recurring_payments/email_admins_no_payment_profile.html"
            try:
                email_content = render_to_string(template_name, 
                                               {'rp':recurring_payment,
                                                'site_display_name': self.site_display_name,
                                                'site_url': self.site_url
                                                })
                self.email.body = email_content
                self.email.content_type = "html"
                self.email.subject = 'Payment method not setup for %s on %s' % ( 
                                    recurring_payment , self.site_display_name)
                
                self.email.send()
            except TemplateDoesNotExist:
                pass

    def email_customer_no_payment_profile(self, recurring_payment):
        """Notify customer that payment method hasn't been setup yet for this recurring payment entry.
        """
        self.email.recipient = recurring_payment.user.email
        if self.email.recipient:
            template_name = "recurring_payments/email_customer_no_payment_profile.html"
            try:
                email_content = render_to_string(template_name, 
                                               {'rp':recurring_payment,
                                                'site_display_name': self.site_display_name,
                                                'site_url': self.site_url
                                                })
                self.email.body = email_content
                self.email.content_type = "html"
                self.email.subject = 'Please update your payment method for %s on %s' % ( 
                                    recurring_payment.description, self.site_display_name)
                
                self.email.send()
            except TemplateDoesNotExist:
                pass
            
    def email_admins_account_disabled(self, recurring_payment, user_by):
        """Notify admin that the recurring payment account is disabled.
        """
        self.email.recipient = self.admin_emails
        if self.email.recipient:
            template_name = "recurring_payments/email_admins_account_disabled.html"
            try:
                email_content = render_to_string(template_name, 
                                               {'rp':recurring_payment,
                                                'user_by': user_by,
                                                'site_display_name': self.site_display_name,
                                                'site_url': self.site_url
                                                })
                self.email.body = email_content
                self.email.content_type = "html"
                self.email.subject = 'Recurring Payment Account (ID:%d) Disabled by %s on %s' % ( 
                       recurring_payment.id, user_by, self.site_display_name)
                
                self.email.send()
            except TemplateDoesNotExist:
                pass
            
            
def run_a_recurring_payment(rp, verbosity=0):
    """
        1) check and populate payment profile for each recurring payment entry.
        2) generate invoice(s) for recurring payments if needed.
        3) make payment transactions for invoice(s) upon due date.
        4) notify admins and customers for after each transaction.
    """
    num_processed = 0
    if rp.status_detail == 'active':
        rp_email_notice = RecurringPaymentEmailNotices()
        now = datetime.now()
        currency_symbol = get_setting('site', 'global', 'currencysymbol')
        
        # check and store payment profiles in local db
        if verbosity > 1:
            print
            print 'Processing for "%s":' % rp
            print '...Populating payment profiles from payment gateway...'
        rp.populate_payment_profile()
        
        # create invoices if needed
        if verbosity > 1:
            print '...Checking and generating invoice(s)  ...' 
        rp.check_and_generate_invoices()

        # look for unpaid invoices with current due date or pass due date
        rp_invoices = RecurringPaymentInvoice.objects.filter(
                                             recurring_payment=rp, 
                                             invoice__balance__gt=0,
                                             billing_dt__lte=now
                                             ).order_by('billing_cycle_start_dt')

        if rp_invoices:
            payment_profiles = PaymentProfile.objects.filter(
                        customer_profile_id=rp.customer_profile_id,
                        status=True,
                        status_detail='active'
                        ).order_by('-update_dt')
            
            if payment_profiles:
        
                for i, rp_invoice in enumerate(rp_invoices):
                    # wait for 3 minutes (duplicate transaction window is 2 minutes) if this is not the first invoice,
                    # otherwise, the payment gateway would through the "duplicate transaction" error.
                    if i > 0: time.sleep(3*60)
                    
                    payment_profile = payment_profiles[0]
                    if rp_invoice.last_payment_failed_dt and \
                        rp_invoice.last_payment_failed_dt > payment_profile.update_dt:
                        # this invoice was processed but failed, and they haven't update the payment profile yet,
                        # so just skip it for now.
                        # only skip if the error code is: E00027 - the transaction was unsuccessful
                        last_error_code = rp_invoice.get_last_transaction_error_code()
                        if last_error_code and last_error_code in UNSUCCESSFUL_TRANS_CODE:
                            continue
                    
                    # make payment transaction and then update recurring_payment fields
                    if verbosity > 1:
                        print '...Making payment transaction for billing cycle (%s -%s) - amount: %s%.2f ...' \
                                % (rp_invoice.billing_cycle_start_dt.strftime('%m-%d-%Y'),
                                   rp_invoice.billing_cycle_end_dt.strftime('%m-%d-%Y'),
                                   currency_symbol, 
                                   rp_invoice.invoice.balance)
                     
                    success = False    
                    
                    payment_profile_id = payment_profile.payment_profile_id
                    payment_transaction = rp_invoice.make_payment_transaction(payment_profile_id)
                    if payment_transaction.status:
                        success = True
                        num_processed += 1
                    
                    
                    if success:
                        rp.last_payment_received_dt = now
                        rp_invoice.payment_received_dt = now
                        rp_invoice.save()
                        rp.num_billing_cycle_completed += 1
                        print '...Success.'
                    else:
                        rp.num_billing_cycle_failed += 1
                        print '...Failed  - \n\t code - %s \n\t text - %s' \
                                            % (payment_transaction.message_code,
                                               payment_transaction.message_text)
                        
                    # send out email notifications - for both successful and failed transactions
                    # to admin
                    rp_email_notice.email_admins_transaction_result(payment_transaction, success=success)
                    # to customer
                    if payment_transaction.message_code not in UNSUCCESSFUL_TRANS_CODE:
                        rp_email_notice.email_customer_transaction_result(payment_transaction)
                    else:
                        # the payment gateway is probably not configured correctly
                        # email to tendenci script support 
                        rp_email_notice.email_script_support_transaction_error(payment_transaction)
            else:
                # email admin - payment profile not set up
                # to admin
                rp_email_notice.email_admins_no_payment_profile(rp)
                # to customer
                rp_email_notice.email_customer_no_payment_profile(rp)
                                 
        
        # calculate the balance by checking for unpaid invoices       
        rp.balance = rp.get_current_balance()
        rp.outstanding_balance = rp.get_outstanding_balance()
        rp.save()
        
    return num_processed  


def api_rp_setup(data): 
    """Create a recurrring payment account. Accepted format: json
    
    Input fields:
        email - required
        description - required
        amount - required
        cp_id - customer profile id, required
        pp_id - customer payment profile id, required
        billing_cycle_start_dt - required
        billing_cycle_end_dt - required
        response_str - required
        login_name 
        login_password
        url 
        first_name
        last_name
          
        
        billing_period - optional, default to 'month'
        billing_frequency - optional, default to 1
        billing_start_dt - optional, default to today
        num_days - optional, default to 0
        has_trial_period - optional, default to False
        trial_period_start_dt - optional, default to today
        trial_period_end_dt - optional, default to today
        trial_amount - optional, default to 0
        
    Output:
        rp_id - a recurring payment id
        rp_url - url to rp
        username        
        result_code
    """
    from decimal import Decimal
    from django.core.validators import email_re
    import dateutil.parser as dparser
    from tendenci.core.imports.utils import get_unique_username
    
    email = data.get('email', '')
    description = data.get('description', '')
    url = data.get('url')
    payment_amount = data.get('amount', '')
    taxable = data.get('taxable', 0)
    if taxable in ('True', 'true', '1', 1): 
        taxable = 1
    else: 
        taxable = 0
    try:
        tax_rate = Decimal(data.get('tax_rate', 0))
        if tax_rate > 1: tax_rate = 0
    except:
        tax_rate = 0
    tax_exempt = data.get('tax_exempt', 0)
    if tax_exempt in ('True', 'true', '1', 1): 
        tax_exempt = 1
    else: 
        tax_exempt = 0
    try:
        payment_amount = Decimal(payment_amount)
    except:
        payment_amount = 0
    cp_id = data.get('cp_id')
    pp_id = data.get('pp_id')
    billing_cycle_start_dt = data.get('billing_cycle_start_dt')
    if billing_cycle_start_dt:
        billing_cycle_start_dt = dparser.parse(billing_cycle_start_dt)
    billing_cycle_end_dt = data.get('billing_cycle_end_dt')
    if billing_cycle_end_dt:
        billing_cycle_end_dt = dparser.parse(billing_cycle_end_dt)
        
    direct_response_str = data.get('response_str')
    
    if not all([email_re.match(email), 
                description,
                payment_amount>0,
                cp_id,
                pp_id,
                billing_cycle_start_dt,
                billing_cycle_end_dt,
                direct_response_str] 
               ):
        return False, {}
    
    # 1) get or create user
    username = data.get('login_name')
    
    # check if user already exists based on email and username
    users = User.objects.filter(email=email, username=username)
    if users:
        u = users[0]
    else:
        # create user account
        u = User()
        u.email=email
        u.username = username
        if not u.username:
            u.username = email.split('@')[0]
        u.username = get_unique_username(u)
        raw_password = data.get('login_password')
        if not raw_password:
            raw_password = User.objects.make_random_password(length=8)
        u.set_password(raw_password)
        u.first_name = data.get('first_name', '')
        u.last_name = data.get('last_name', '')
        u.is_staff = False
        u.is_superuser = False
        u.save()
        
        profile = Profile.objects.create(
           user=u, 
           creator=u, 
           creator_username=u.username,
           owner=u, 
           owner_username=u.username, 
           email=u.email
        )
        
    # 2) create a recurring payment account
    rp = RecurringPayment()
    rp.user = u
    rp.description = description
    rp.url = url
    rp.payment_amount = payment_amount
    rp.taxable = taxable
    rp.tax_rate = tax_rate
    rp.tax_exempt = tax_exempt
    rp.customer_profile_id = cp_id
    rp.billing_start_dt = billing_cycle_start_dt
    
    has_trial_period = data.get('has_trial_period')
    trial_period_start_dt = data.get('trial_period_start_dt')
    trial_period_end_dt = data.get('trial_period_end_dt')
    if has_trial_period in ['True', '1',  True, 1] and all([trial_period_start_dt,
                                                            trial_period_end_dt]):
        rp.has_trial_period = True
        rp.trial_period_start_dt = dparser.parse(trial_period_start_dt) 
        rp.trial_period_end_dt = dparser.parse(trial_period_end_dt)
    else:
        rp.has_trial_period = False
        
    rp.status_detail = 'active'
    rp.save()
    
    # 3) create a payment profile account
    payment_profile_exists = PaymentProfile.objects.filter(
                                        customer_profile_id=cp_id,
                                        payment_profile_id=pp_id
                                        ).exists()
    if not payment_profile_exists:
        PaymentProfile.objects.create(
                        customer_profile_id=cp_id,
                        payment_profile_id=pp_id,
                        owner=u,
                        owner_username=u.username
                        )
        
    # 4) create rp invoice
    billing_cycle = {'start': billing_cycle_start_dt,
                     'end': billing_cycle_end_dt}
    rp_invoice = rp.create_invoice(billing_cycle, billing_cycle_start_dt)
    rp_invoice.invoice.tender(rp.user)
    
    # 5) create rp transaction
    now = datetime.now()
    payment = Payment()
    payment.payments_pop_by_invoice_user(rp.user, 
                                         rp_invoice.invoice, 
                                         rp_invoice.invoice.guid)
    payment_transaction = PaymentTransaction(
                                    recurring_payment=rp,
                                    recurring_payment_invoice=rp_invoice,
                                    payment_profile_id=pp_id,
                                    trans_type='auth_capture',
                                    amount=rp_invoice.invoice.total,
                                    status=True)
    payment = payment_update_from_response(payment, direct_response_str)
    payment.mark_as_paid()
    payment.save()
    rp_invoice.invoice.make_payment(rp.user, Decimal(payment.amount))
    rp_invoice.invoice.save()
    
    
    rp_invoice.payment_received_dt = now
    rp_invoice.save()
    rp.last_payment_received_dt = now
    rp.num_billing_cycle_completed += 1
    rp.save()
    
    payment_transaction.payment = payment
    payment_transaction.result_code = data.get('result_code')
    payment_transaction.message_code = data.get('message_code')
    payment_transaction.message_text = data.get('message_text')
        
    payment_transaction.save()
    
    site_url = get_setting('site', 'global', 'siteurl')

    
    return True, {'rp_id': rp.id,
                  'rp_url': '%s%s' %  (site_url, 
                                reverse('recurring_payment.view_account', args=[rp.id])),
                  'username': rp.user.username}

def api_add_rp(data):
    """Create a recurrring payment account. Accepted format: json
    
    Input fields:
        email - required
        description - required
        payment_amount - required
        billing_period - optional, default to 'month'
        billing_frequency - optional, default to 1
        billing_start_dt - optional, default to today
        num_days - optional, default to 0
        has_trial_period - optional, default to False
        trial_period_start_dt - optional, default to today
        trial_period_end_dt - optional, default to today
        trial_amount - optional, default to 0
        
    Output:
        rp_id - a recurring payment id
        result_code
    """
    ALLOWED_FIELES = ('email',
                      'description',
                      'payment_amount',
                      'billing_period',
                      'billing_frequency',
                      'billing_start_dt',
                      'num_days',
                      'has_trial_period',
                      'trial_period_start_dt',
                      'trial_period_end_dt',
                      'trial_amount',
                      )
    from decimal import Decimal
    from django.core.validators import email_re
    import dateutil.parser as dparser
    from tendenci.core.imports.utils import get_unique_username
    
    email = data.get('email', '')
    payment_amount = data.get('payment_amount', '')
    try:
        payment_amount = Decimal(payment_amount)
    except:
        payment_amount = 0
    
    
    if not all([email_re.match(email), 
                data.has_key('description'),
                payment_amount>0]):
        return False, {}
    
    
    rp = RecurringPayment()
    for key, value in data.items():
        if key in ALLOWED_FIELES:
            if hasattr(rp, key):
                setattr(rp, key, value)
            
    if rp.billing_start_dt:
        try:
            rp.billing_start_dt = dparser.parse(rp.billing_start_dt)
        except:
            rp.billing_start_dt = datetime.now()
    else:
        rp.billing_start_dt = datetime.now()
    if rp.trial_period_start_dt:
        try:
            rp.trial_period_start_dt = dparser.parse(rp.trial_period_start_dt)
        except:
            rp.trial_period_start_dt = datetime.now()
    
    if rp.trial_period_end_dt:
        try:
            rp.trial_period_end_dt = dparser.parse(rp.trial_period_end_dt)
        except:
            rp.trial_period_end_dt = datetime.now()
        
    rp.payment_amount = Decimal(rp.payment_amount)
    
    try:
        rp.billing_frequency = int(rp.billing_frequency)
    except:
        rp.billing_frequency = 1
    try:
        rp.num_days = int(rp.num_days)
    except:
        rp.num_days = 1
    if rp.has_trial_period in ['True', '1',  True, 1] and all([rp.trial_period_start_dt,
                                                              rp.trial_period_end_dt]):
        rp.has_trial_period = True
    else:
        rp.has_trial_period = False
        
    # start the real work
    
#    # get or create a user account with this email
#    users = User.objects.filter(email=email)
#    if users:
#        u = users[0]
#    else:

    # always create a new user account - This is very important!
    # it is to prevent hacker from trying to use somebody else's account. 
    u = User()
    u.email=email
    u.username = data.get('username', '')
    if not u.username:
        u.username = email.split('@')[0]
    u.username = get_unique_username(u)
    raw_password = data.get('password', '')
    if not raw_password:
        raw_password = User.objects.make_random_password(length=8)
    u.set_password(raw_password)
    u.first_name = data.get('first_name', '')
    u.last_name = data.get('last_name', '')
    u.is_staff = False
    u.is_superuser = False
    u.save()
    
#    profile = Profile.objects.create(
#           user=u, 
#           creator=u, 
#           creator_username=u.username,
#           owner=u, 
#           owner_username=u.username, 
#           email=u.email
#        )
    
    # add a recurring payment entry for this user
    rp.user = u
    # activate it when payment info is received
    rp.status_detail = 'inactive'
    rp.save()
    
    return True, {'rp_id': rp.id}
        
        
def api_get_rp_token(data):
    """Get the token for using authorize.net hosted profile page
        Accepted format: json
    
    Input fields:
        rp_id - required
        iframe_communicator_url
        
    Output:
        token
        gateway_error
        payment_profile_id
        result_code
    """
    rp_id = data.get('rp_id', 0)
    iframe_communicator_url = data.get('iframe_communicator_url', '')
    
    try:
        rp = RecurringPayment.objects.get(id=int(rp_id))
    except:
        return False, {}
    
    token, gateway_error = get_token(rp, CIMCustomerProfile, 
                                     CIMHostedProfilePage, 
                                     iframe_communicator_url)
    
    d = {'token': token,
         'gateway_error': gateway_error}
    
    # also pass the payment_profile_id
    payment_profiles = PaymentProfile.objects.filter(customer_profile_id=rp.customer_profile_id, 
                                                    status=True, 
                                                    status_detail='active')
    if payment_profiles:
        payment_profile_id = (payment_profiles[0]).payment_profile_id
    else:
        payment_profile_id = ''
        
    d['payment_profile_id'] = payment_profile_id
    
    
    if gateway_error:
        status = False
    else:
        status = True
        
    return status, d


def api_verify_rp_payment_profile(data):
    """Verify if this recurring payment account
        has a valid payment profile.
        Accepted format: json
    
    Input fields:
        rp_id - required
        
    Output:
        has_payment_profile
        result_code
    """
    rp_id = data.get('rp_id', 0)
    
    try:
        rp = RecurringPayment.objects.get(id=int(rp_id))
    except:
        return False, {}
    
    d = {}
    pay_now = data.get('pay_now', '')
    if pay_now == 'yes': pay_now = True
    else: pay_now = False
    
    # pp - customer payment_profile
    validation_mode=''
    if not pay_now: 
        validation_mode='liveMode'
        
    is_valid = True
        
    valid_cpp_ids, invalid_cpp_ids = rp.populate_payment_profile(validation_mode=validation_mode)
    if valid_cpp_ids:
        d['valid_cpp_id'] = valid_cpp_ids[0]      
        
        if pay_now:
            # make a transaction NOW
            billing_cycle = {'start': rp.billing_start_dt, 
                             'end': rp.billing_start_dt + relativedelta(months=rp.billing_frequency)}
            billing_dt = datetime.now()
            rp_invoice = rp.create_invoice(billing_cycle, billing_dt)
            payment_transaction = rp_invoice.make_payment_transaction(d['valid_cpp_id'])
            if not payment_transaction.status:
                # payment failed
                rp.num_billing_cycle_failed += 1
                rp.save()
                d['invalid_cpp_id'] = d['valid_cpp_id']
                d['valid_cpp_id'] = ''
                is_valid = False
            else:
                # success
                
                # update rp and rp_invoice
                if rp.status_detail <> 'active':
                    rp.status_detail = 'active'
                
                now = datetime.now()
                rp.last_payment_received_dt = now
                rp.num_billing_cycle_completed += 1
                rp.save()
                rp_invoice.payment_received_dt = now
                rp_invoice.save()
                
                
                # send out the invoice view page
                d['receipt_url'] = '%s%s' % (get_setting('site', 'global', 'siteurl'), 
                                             reverse('recurring_payment.transaction_receipt', 
                                                args=[rp.id, 
                                                payment_transaction.id,
                                                rp.guid]))
                
                # email to user
                rp_email_notice = RecurringPaymentEmailNotices()
                rp_email_notice.email_customer_transaction_result(payment_transaction)
                
            
    
    if invalid_cpp_ids:
        d['invalid_cpp_id']= invalid_cpp_ids[0]
                  
    return is_valid, d


    
        
    
    
        
    
    
    
            
    


from django.contrib import admin
from django.core.urlresolvers import reverse

from tendenci.core.perms.admin import TendenciBaseModelAdmin
from tendenci.apps.stories.models import Story
from tendenci.apps.stories.forms import StoryAdminForm
from tendenci.core.event_logs.models import EventLog
from tendenci.core.perms.utils import update_perms_and_save


class StoryAdmin(TendenciBaseModelAdmin):
    list_display = ('image_preview', 'title', 'tags', 'status', 'ncsortorder')
    search_fields = ('title', 'content')
    list_editable = ['title', 'tags', 'ncsortorder']
    fieldsets = [('Story Information', {
                      'fields': ['title',
                                 'content',
                                 'photo_upload',
                                 'full_story_link',
                                 'link_title',
                                 'tags',
                                 'start_dt',
                                 'end_dt',
                                 'expires'
                                 ],
                      }),
                    ('Permissions', {'fields': ('allow_anonymous_view',)}),
                    ('Advanced Permissions', {'classes': ('collapse',), 'fields': (
                        'user_perms',
                        'member_perms',
                        'group_perms',
                    )}),
                     ('Administrator Only', {
                      'fields': ['syndicate',
                                 'status',
                                 'status_detail'],
                      'classes': ['admin-only'],
                    })]
    form = StoryAdminForm
    ordering = ['-ncsortorder']

    class Media:
        css = {
            "all": ("css/websymbols.css",)
        }
        js = (
            'js/jquery-1.6.2.min.js',
            'js/jquery-ui-1.8.17.custom.min.js',
            'js/story-admin-list-reorder.js',
            'js/global/tinymce.event_handlers.js',
        )
    
    def save_model(self, request, object, form, change):
        story = form.save(commit=False)

        # save photo
        if 'photo_upload' in form.cleaned_data:
            photo = form.cleaned_data['photo_upload']
            if photo:
                story.save(photo=photo)

        story = update_perms_and_save(request, form, story)

        log_defaults = {
            'instance': object,
            'action': "edit"
        }
        if not change:
            log_defaults['action'] = "add"

        # Handle a special case for bulk reordering via the list view.
        if form.changed_data != ['ncsortorder']:
            EventLog.objects.log(**log_defaults)
        return object

    def image_preview(self, obj):
        if obj.image:
            args = [obj.image.pk]
            args.append("100x50")
            args.append("crop")
            return '<img src="%s" />' % reverse('file', args=args)
        else:
            return "No image"
    image_preview.allow_tags = True
    image_preview.short_description = 'Image'

admin.site.register(Story, StoryAdmin)



from django import template
from django.conf import settings
from django.core.urlresolvers import reverse
from django.template import TemplateSyntaxError, Variable

register = template.Library()

class MobileLinkNode(template.Node):
    def __init__(self, redirect_url, link_name):
        self.redirect_url = redirect_url
        self.link_name = link_name
    
    def render(self, context):
        try:
            redirect_url = Variable(self.redirect_url)
            redirect_url = redirect_url.resolve(context)
        except:
            redirect_url = self.redirect_url

        return "<a href='%s?next=%s'>%s</a>" % (
            reverse('toggle_mobile_mode'),
            redirect_url,
            self.link_name)

@register.tag(name="toggle_mobile_link")
def toggle_mobile_link(parser, token):
    """
    {% toggle_mobile_link request.get_full_path "See mobile" %}
    """

    bits = token.split_contents()
    
    if len(bits) > 3:
        message = "'%s' tag requires exactly 2 arguments" % bits[0]
        raise TemplateSyntaxError(message)
        
    return MobileLinkNode(bits[1], bits[2][1:-1])



from django.views.generic import TemplateView
from django.conf import settings


class NewsletterGeneratorView(TemplateView):
    template_name="newsletters/newsletter_generator.html"

    def get_context_data(self, **kwargs):
        context = super(NewsletterGeneratorView, self).get_context_data(**kwargs)
        cm_api_key = getattr(settings, 'CAMPAIGNMONITOR_API_KEY', None) 
        cm_client_id = getattr(settings, 'CAMPAIGNMONITOR_API_CLIENT_ID', None)
        if cm_api_key and cm_client_id:
            context['CAMPAIGNMONITOR_ENABLED'] = True
        else:
            context['CAMPAIGNMONITOR_ENABLED'] = False

        return context


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'NoticeType'
        db.create_table('notifications_noticetype', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('label', self.gf('django.db.models.fields.CharField')(max_length=40)),
            ('display', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('description', self.gf('django.db.models.fields.CharField')(max_length=100)),
            ('default', self.gf('django.db.models.fields.IntegerField')()),
        ))
        db.send_create_signal('notifications', ['NoticeType'])

        # Adding model 'NoticeSetting'
        db.create_table('notifications_noticesetting', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('notice_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['notifications.NoticeType'])),
            ('medium', self.gf('django.db.models.fields.CharField')(max_length=1)),
            ('send', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('notifications', ['NoticeSetting'])

        # Adding unique constraint on 'NoticeSetting', fields ['user', 'notice_type', 'medium']
        db.create_unique('notifications_noticesetting', ['user_id', 'notice_type_id', 'medium'])

        # Adding model 'Notice'
        db.create_table('notifications_notice', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('message', self.gf('django.db.models.fields.TextField')()),
            ('notice_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['notifications.NoticeType'])),
            ('added', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('unseen', self.gf('django.db.models.fields.BooleanField')(default=True)),
            ('archived', self.gf('django.db.models.fields.BooleanField')(default=False)),
            ('on_site', self.gf('django.db.models.fields.BooleanField')(default=False)),
        ))
        db.send_create_signal('notifications', ['Notice'])

        # Adding model 'NoticeQueueBatch'
        db.create_table('notifications_noticequeuebatch', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('pickled_data', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal('notifications', ['NoticeQueueBatch'])

        # Adding model 'NoticeEmail'
        db.create_table('notifications_noticeemail', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('guid', self.gf('django.db.models.fields.CharField')(max_length=50)),
            ('sender', self.gf('django.db.models.fields.CharField')(max_length=200, blank=True)),
            ('emails', self.gf('django.db.models.fields.CharField')(max_length=200, blank=True)),
            ('bcc', self.gf('django.db.models.fields.CharField')(max_length=200, blank=True)),
            ('notice_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['notifications.NoticeType'])),
            ('reply_to', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('from_display', self.gf('django.db.models.fields.CharField')(max_length=100, blank=True)),
            ('title', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('content', self.gf('django.db.models.fields.TextField')(blank=True)),
            ('content_type', self.gf('django.db.models.fields.CharField')(max_length=10)),
            ('date_sent', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
        ))
        db.send_create_signal('notifications', ['NoticeEmail'])

        # Adding model 'ObservedItem'
        db.create_table('notifications_observeditem', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['auth.User'])),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('notice_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['notifications.NoticeType'])),
            ('added', self.gf('django.db.models.fields.DateTimeField')(default=datetime.datetime.now)),
            ('signal', self.gf('django.db.models.fields.TextField')()),
        ))
        db.send_create_signal('notifications', ['ObservedItem'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'NoticeSetting', fields ['user', 'notice_type', 'medium']
        db.delete_unique('notifications_noticesetting', ['user_id', 'notice_type_id', 'medium'])

        # Deleting model 'NoticeType'
        db.delete_table('notifications_noticetype')

        # Deleting model 'NoticeSetting'
        db.delete_table('notifications_noticesetting')

        # Deleting model 'Notice'
        db.delete_table('notifications_notice')

        # Deleting model 'NoticeQueueBatch'
        db.delete_table('notifications_noticequeuebatch')

        # Deleting model 'NoticeEmail'
        db.delete_table('notifications_noticeemail')

        # Deleting model 'ObservedItem'
        db.delete_table('notifications_observeditem')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2012, 9, 11, 12, 30, 52, 980565)'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2012, 9, 11, 12, 30, 52, 980457)'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'notifications.notice': {
            'Meta': {'ordering': "['-added']", 'object_name': 'Notice'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'archived': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'message': ('django.db.models.fields.TextField', [], {}),
            'notice_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['notifications.NoticeType']"}),
            'on_site': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'unseen': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'notifications.noticeemail': {
            'Meta': {'object_name': 'NoticeEmail'},
            'bcc': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'content': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'content_type': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'date_sent': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'emails': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'from_display': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'guid': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'notice_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['notifications.NoticeType']"}),
            'reply_to': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'sender': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'title': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        },
        'notifications.noticequeuebatch': {
            'Meta': {'object_name': 'NoticeQueueBatch'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'pickled_data': ('django.db.models.fields.TextField', [], {})
        },
        'notifications.noticesetting': {
            'Meta': {'unique_together': "(('user', 'notice_type', 'medium'),)", 'object_name': 'NoticeSetting'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'medium': ('django.db.models.fields.CharField', [], {'max_length': '1'}),
            'notice_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['notifications.NoticeType']"}),
            'send': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'notifications.noticetype': {
            'Meta': {'object_name': 'NoticeType'},
            'default': ('django.db.models.fields.IntegerField', [], {}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'display': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '40'})
        },
        'notifications.observeditem': {
            'Meta': {'ordering': "['-added']", 'object_name': 'ObservedItem'},
            'added': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'notice_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['notifications.NoticeType']"}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'signal': ('django.db.models.fields.TextField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        }
    }

    complete_apps = ['notifications']



import os

from django import forms
from django.conf import settings
from django.utils.translation import ugettext_lazy as _

from tendenci.apps.user_groups.importer.models import ImportFile

class UploadForm(forms.ModelForm):
    """
    CSV upload form for imports
    """
    file = forms.FileField(label=_("XLS File"), help_text=_('The importer will automatically recognize fields from your XLS spreadsheet. A subscribers First name is found by looking for the word "first" in the heading. "Last" and "full" are also automatically matched. Email fields are matched by finding a valid email address in them. Records without a valid email address will not be imported.'))
    
    class Meta:
        model = ImportFile
        fields = (
            'file',
        )



# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Adding model 'GroupSubscription'
        db.create_table('subscribers_groupsubscription', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('group', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['user_groups.Group'])),
            ('subscriber', self.gf('django.db.models.fields.related.ForeignKey')(related_name='subscriptions', null=True, to=orm['forms.FormEntry'])),
            ('create_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('update_dt', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal('subscribers', ['GroupSubscription'])

        # Adding unique constraint on 'GroupSubscription', fields ['group', 'subscriber']
        db.create_unique('subscribers_groupsubscription', ['group_id', 'subscriber_id'])

        # Adding model 'SubscriberData'
        db.create_table('subscribers_subscriberdata', (
            ('id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('subscription', self.gf('django.db.models.fields.related.ForeignKey')(related_name='data', to=orm['subscribers.GroupSubscription'])),
            ('field_label', self.gf('django.db.models.fields.CharField')(max_length=2000)),
            ('value', self.gf('django.db.models.fields.CharField')(max_length=2000)),
        ))
        db.send_create_signal('subscribers', ['SubscriberData'])

        # Adding unique constraint on 'SubscriberData', fields ['subscription', 'field_label']
        db.create_unique('subscribers_subscriberdata', ['subscription_id', 'field_label'])


    def backwards(self, orm):
        
        # Removing unique constraint on 'SubscriberData', fields ['subscription', 'field_label']
        db.delete_unique('subscribers_subscriberdata', ['subscription_id', 'field_label'])

        # Removing unique constraint on 'GroupSubscription', fields ['group', 'subscriber']
        db.delete_unique('subscribers_groupsubscription', ['group_id', 'subscriber_id'])

        # Deleting model 'GroupSubscription'
        db.delete_table('subscribers_groupsubscription')

        # Deleting model 'SubscriberData'
        db.delete_table('subscribers_subscriberdata')


    models = {
        'auth.group': {
            'Meta': {'object_name': 'Group'},
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'ordering': "('content_type__app_label', 'content_type__model', 'codename')", 'unique_together': "(('content_type', 'codename'),)", 'object_name': 'Permission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'Meta': {'object_name': 'User'},
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2012, 9, 14, 9, 46, 8, 911266)'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'symmetrical': 'False', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime(2012, 9, 14, 9, 46, 8, 911138)'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'symmetrical': 'False', 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'entities.entity': {
            'Meta': {'object_name': 'Entity'},
            'admin_notes': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'allow_anonymous_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_anonymous_view': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_member_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_member_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'contact_name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'entity_creator'", 'to': "orm['auth.User']"}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'email': ('django.db.models.fields.CharField', [], {'max_length': '120', 'blank': 'True'}),
            'entity_name': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'entity_parent_id': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'entity_type': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'fax': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'guid': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'notes': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'entity_owner'", 'to': "orm['auth.User']"}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'phone': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'summary': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'website': ('django.db.models.fields.CharField', [], {'max_length': '300', 'blank': 'True'})
        },
        'forms.form': {
            'Meta': {'object_name': 'Form'},
            'allow_anonymous_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_anonymous_view': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_member_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_member_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'completion_url': ('django.db.models.fields.URLField', [], {'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'forms_form_creator'", 'to': "orm['auth.User']"}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'custom_payment': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'email_copies': ('django.db.models.fields.CharField', [], {'max_length': '200', 'blank': 'True'}),
            'email_from': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'email_text': ('django.db.models.fields.TextField', [], {'default': "''", 'max_length': '2000', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'intro': ('django.db.models.fields.TextField', [], {'max_length': '2000', 'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'forms_form_owner'", 'to': "orm['auth.User']"}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'payment_methods': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['payments.PaymentMethod']", 'symmetrical': 'False', 'blank': 'True'}),
            'response': ('django.db.models.fields.TextField', [], {'max_length': '2000', 'blank': 'True'}),
            'send_email': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'slug': ('django.db.models.fields.SlugField', [], {'unique': 'True', 'max_length': '100', 'db_index': 'True'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'subject_template': ('django.db.models.fields.CharField', [], {'default': "'[title] - [first name]  [last name] - [phone]'", 'max_length': '200', 'null': 'True', 'blank': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'forms.formentry': {
            'Meta': {'object_name': 'FormEntry'},
            'entry_path': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '200', 'blank': 'True'}),
            'entry_time': ('django.db.models.fields.DateTimeField', [], {}),
            'form': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'entries'", 'to': "orm['forms.Form']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'payment_method': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['payments.PaymentMethod']", 'null': 'True'}),
            'pricing': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['forms.Pricing']", 'null': 'True'})
        },
        'forms.pricing': {
            'Meta': {'object_name': 'Pricing'},
            'form': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['forms.Form']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'price': ('django.db.models.fields.DecimalField', [], {'null': 'True', 'max_digits': '10', 'decimal_places': '2', 'blank': 'True'})
        },
        'payments.paymentmethod': {
            'Meta': {'object_name': 'PaymentMethod'},
            'admin_only': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'human_name': ('django.db.models.fields.CharField', [], {'max_length': '200'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_online': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'machine_name': ('django.db.models.fields.CharField', [], {'max_length': '200'})
        },
        'perms.objectpermission': {
            'Meta': {'object_name': 'ObjectPermission'},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['user_groups.Group']", 'null': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.IntegerField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']", 'null': 'True'})
        },
        'subscribers.groupsubscription': {
            'Meta': {'unique_together': "(('group', 'subscriber'),)", 'object_name': 'GroupSubscription'},
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['user_groups.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'subscriber': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'subscriptions'", 'null': 'True', 'to': "orm['forms.FormEntry']"}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'subscribers.subscriberdata': {
            'Meta': {'unique_together': "(('subscription', 'field_label'),)", 'object_name': 'SubscriberData'},
            'field_label': ('django.db.models.fields.CharField', [], {'max_length': '2000'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'subscription': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'data'", 'to': "orm['subscribers.GroupSubscription']"}),
            'value': ('django.db.models.fields.CharField', [], {'max_length': '2000'})
        },
        'user_groups.group': {
            'Meta': {'object_name': 'Group'},
            'allow_anonymous_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_anonymous_view': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_member_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_member_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_self_add': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_self_remove': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'allow_user_edit': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'allow_user_view': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'auto_respond': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'auto_respond_priority': ('django.db.models.fields.FloatField', [], {'default': '0', 'blank': 'True'}),
            'auto_respond_template': ('django.db.models.fields.CharField', [], {'max_length': '100', 'blank': 'True'}),
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'user_groups_group_creator'", 'to': "orm['auth.User']"}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'description': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'email_recipient': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'entity': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['entities.Entity']", 'null': 'True', 'blank': 'True'}),
            'guid': ('django.db.models.fields.CharField', [], {'max_length': '40'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255', 'blank': 'True'}),
            'members': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.User']", 'through': "orm['user_groups.GroupMembership']", 'symmetrical': 'False'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255'}),
            'notes': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'owner': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'user_groups_group_owner'", 'to': "orm['auth.User']"}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'symmetrical': 'False', 'related_name': "'group_permissions'", 'blank': 'True', 'to': "orm['auth.Permission']"}),
            'show_as_option': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'slug': ('tendenci.core.base.fields.SlugField', [], {'unique': 'True', 'max_length': '100', 'db_index': 'True'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'type': ('django.db.models.fields.CharField', [], {'default': "'distribution'", 'max_length': '75', 'blank': 'True'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        'user_groups.groupmembership': {
            'Meta': {'unique_together': "(('group', 'member'),)", 'object_name': 'GroupMembership'},
            'create_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'creator_id': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'creator_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['user_groups.Group']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'member': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'group_member'", 'to': "orm['auth.User']"}),
            'owner_id': ('django.db.models.fields.IntegerField', [], {'default': '0'}),
            'owner_username': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'role': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255', 'blank': 'True'}),
            'sort_order': ('django.db.models.fields.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'status': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'status_detail': ('django.db.models.fields.CharField', [], {'default': "'active'", 'max_length': '50'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['subscribers']



from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('tendenci.core.email_blocks.views',
    url(r'^$', 'search', name="email_blocks"),
    url(r'^(?P<id>\d+)/$', 'view', name="email_block.view"),
    url(r'^search/$', 'search', name="email_block.search"),
    url(r'^add/$', 'add', name="email_block.add"),
    url(r'^edit/(?P<id>\d+)/$', 'edit', name="email_block.edit"),
    url(r'^delete/(?P<id>\d+)/$', 'delete', name="email_block.delete"),
)


import os
import csv

from django.template.defaultfilters import slugify
from django.core.files.storage import default_storage

from tendenci.core.base.utils import normalize_newline

def geocode_api(**kwargs):
    import simplejson, urllib
    GEOCODE_BASE_URL = 'http://maps.googleapis.com/maps/api/geocode/json'
    kwargs['sensor'] = kwargs.get('sensor', 'false')
    url = '%s?%s' % (GEOCODE_BASE_URL, urllib.urlencode(kwargs))
    return simplejson.load(urllib.urlopen(url))

def get_coordinates(address):
    """
    Get the latitude and longitude for the address parameter.
    Return a 2-tuple with latitude and longitude.
    Else return a 2-tuple with None Type objects.
    """
    result = geocode_api(address=address)

    if result['status'] == 'OK':
        return result['results'][0]['geometry']['location'].values()
    
    return (None, None)

def distance_api(*args, **kwargs):
    import simplejson, urllib

    output = kwargs.get('output', 'json')
    distance_base_url = 'http://maps.googleapis.com/maps/api/distancematrix/%s' & ouput

    kwargs.update({
        'origins':kwargs.get('origins',''),
        'destinations': kwargs.get('destinations',''), 
        'sensor': kwargs.get('sensor', 'false')
    })

    url = '%s?%s' % (distance_base_url, urllib.urlencode(kwargs))
    return simplejson.load(urllib.urlopen(url))
 
def distance_via_sphere(lat1, long1, lat2, long2):
    """
    http://www.johndcook.com/python_longitude_latitude.html
    Distance in miles multiply by 3960
    Distance in kilometers multiply by 6373
    """
    import math

    # Convert latitude and longitude to 
    # spherical coordinates in radians.
    degrees_to_radians = math.pi/180.0
        
    # phi = 90 - latitude
    phi1 = (90.0 - lat1)*degrees_to_radians
    phi2 = (90.0 - lat2)*degrees_to_radians
        
    # theta = longitude
    theta1 = long1*degrees_to_radians
    theta2 = long2*degrees_to_radians
        
    # Compute spherical distance from spherical coordinates.
        
    # For two locations in spherical coordinates 
    # (1, theta, phi) and (1, theta, phi)
    # cosine( arc length ) = 
    #    sin phi sin phi' cos(theta-theta') + cos phi cos phi'
    # distance = rho * arc length
    
    cos = (math.sin(phi1)*math.sin(phi2)*math.cos(theta1 - theta2) + 
           math.cos(phi1)*math.cos(phi2))
    arc = math.acos( cos )

    # Remember to multiply arc by the radius of the earth 
    # in your favorite set of units to get length.
    return arc * 3960

def csv_to_dict(file_path, **kwargs):
    """
    Returns a list of dicts. Each dict represents record.
    """
    machine_name = kwargs.get('machine_name', False)

    # null byte; assume xls; not csv
    if has_null_byte(file_path):
        return []

    normalize_newline(file_path)
    csv_file = csv.reader(default_storage.open(file_path, 'rU'))
    colnames = csv_file.next()  # row 1;

    if machine_name:
        colnames = [slugify(c).replace('-', '') for c in colnames]
        
    cols = xrange(len(colnames))
    lst = []
    
    # make sure colnames are unique
    duplicates = {}
    for i in cols:
        for j in cols:
            # compare with previous and next fields
            if i != j and colnames[i] == colnames[j]:
                number = duplicates.get(colnames[i], 0) + 1
                duplicates[colnames[i]] = number
                colnames[j] = colnames[j] + "-" + str(number)
    
    for row in csv_file:
        entry = {}
        rows = len(row) - 1
        for col in cols:
            if col > rows:
                break  # go to next row
            entry[colnames[col]] = row[col]
        lst.append(entry)

    return lst  # list of dictionaries

def has_null_byte(file_path):
    f = default_storage.open(file_path, 'r')
    data = f.read()
    f.close()
    return ('\0' in data)


# encoding: utf-8
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models

class Migration(SchemaMigration):

    def forwards(self, orm):
        
        # Changing field 'Setting.input_value'
        db.alter_column('site_settings_setting', 'input_value', self.gf('django.db.models.fields.CharField')(max_length=1000))


    def backwards(self, orm):
        
        # Changing field 'Setting.input_value'
        db.alter_column('site_settings_setting', 'input_value', self.gf('django.db.models.fields.CharField')(max_length=255))


    models = {
        'site_settings.setting': {
            'Meta': {'object_name': 'Setting'},
            'client_editable': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'data_type': ('django.db.models.fields.CharField', [], {'max_length': '10'}),
            'default_value': ('django.db.models.fields.TextField', [], {'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'input_type': ('django.db.models.fields.CharField', [], {'max_length': '25'}),
            'input_value': ('django.db.models.fields.CharField', [], {'max_length': '1000', 'blank': 'True'}),
            'is_secure': ('django.db.models.fields.BooleanField', [], {'default': 'False'}),
            'label': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'parent_id': ('django.db.models.fields.IntegerField', [], {'default': '0', 'blank': 'True'}),
            'scope': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'scope_category': ('django.db.models.fields.CharField', [], {'max_length': '50'}),
            'store': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'update_dt': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'null': 'True', 'blank': 'True'}),
            'updated_by': ('django.db.models.fields.CharField', [], {'max_length': '50', 'blank': 'True'}),
            'value': ('django.db.models.fields.TextField', [], {'blank': 'True'})
        }
    }

    complete_apps = ['site_settings']



from django import forms
from tendenci.apps.pluginmanager.models import PluginApp
from tendenci.apps.pluginmanager.utils import plugin_options


class PluginAppForm(forms.ModelForm):
    package = forms.ChoiceField(widget=forms.Select(), label="App Name")

    class Meta:
        model = PluginApp
        fields = (
            'package',
            'description',
            'is_enabled',
        )

    def __init__(self, *args, **kwargs):
        super(PluginAppForm, self).__init__(*args, **kwargs)
        if self.instance:
            self.fields['package'].choices = ((self.instance.package, self.instance.title),)
        else:
            self.fields['package'].choices = plugin_options()

    def clean_package(self):
        package = self.cleaned_data['package']
        package = package.strip().replace(' ', '_')

        try:
            __import__(package)
        except:
            raise forms.ValidationError('This plugin does not exist. Please add the name of a valid plugin.')

        return package
